# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Common
PROJECT_ID_PROP = "project.id"

# Data
INPUT_DELIMITER = "delimiter"
INPUT_COMPRESSION = "compression"
COMPRESSION_BZIP2 = "bzip2"
COMPRESSION_GZIP = "gzip"
COMPRESSION_DEFLATE = "deflate"
COMPRESSION_LZ4 = "lz4"
COMPRESSION_NONE = "None"
FORMAT_JSON = "json"
FORMAT_CSV = "csv"
FORMAT_DELTA = "delta"
FORMAT_TXT = "txt"
FORMAT_AVRO = "avro"
FORMAT_PRQT = "parquet"
FORMAT_AVRO_EXTD = "com.databricks.spark.avro"
FORMAT_BIGQUERY = "com.google.cloud.spark.bigquery"
FORMAT_JDBC = "jdbc"
FORMAT_REDSHIFT = "io.github.spark_redshift_community.spark.redshift"
JDBC_URL = "url"
JDBC_TABLE = "dbtable"
JDBC_QUERY = "query"
JDBC_DRIVER = "driver"
JDBC_FETCHSIZE = "fetchsize"
JDBC_BATCH_SIZE = "batchsize"
JDBC_PARTITIONCOLUMN = "partitionColumn"
JDBC_LOWERBOUND = "lowerBound"
JDBC_UPPERBOUND = "upperBound"
JDBC_NUMPARTITIONS = "numPartitions"
JDBC_SESSIONINITSTATEMENT = "sessionInitStatement"
JDBC_CREATE_TABLE_OPTIONS = "createTableOptions"
CSV_CHARTOESCAPEQUOTEESCAPING = "charToEscapeQuoteEscaping"
CSV_COLUMNNAMEOFCORRUPTRECORD = "columnNameOfCorruptRecord"
CSV_COMMENT = "comment"
CSV_COMPRESSION = "compression"
CSV_DATEFORMAT = "dateFormat"
CSV_EMPTYVALUE = "emptyValue"
CSV_ENCODING = "encoding"
CSV_ENFORCESCHEMA = "enforceSchema"
CSV_ESCAPE = "escape"
CSV_ESCAPEQUOTES = "escapeQuotes"
CSV_HEADER = "header"
CSV_IGNORELEADINGWHITESPACE = "ignoreLeadingWhiteSpace"
CSV_IGNORETRAILINGWHITESPACE = "ignoreTrailingWhiteSpace"
CSV_INFER_SCHEMA = "inferSchema"
CSV_LINESEP = "lineSep"
CSV_LOCALE = "locale"
CSV_MAXCHARSPERCOLUMN = "maxCharsPerColumn"
CSV_MAXCOLUMNS = "maxColumns"
CSV_MODE = "mode"
CSV_MULTILINE = "multiLine"
CSV_NANVALUE = "nanValue"
CSV_NULLVALUE = "nullValue"
CSV_NEGATIVEINF = "negativeInf"
CSV_POSITIVEINF = "positiveInf"
CSV_QUOTE = "quote"
CSV_QUOTEALL = "quoteAll"
CSV_SAMPLINGRATIO = "samplingRatio"
CSV_SEP = "sep"
CSV_TIMESTAMPFORMAT = "timestampFormat"
CSV_TIMESTAMPNTZFORMAT = "timestampNTZFormat"
CSV_UNESCAPEDQUOTEHANDLING = "unescapedQuoteHandling"
FORMAT_HBASE = "org.apache.hadoop.hbase.spark"
TABLE = "table"
TEMP_GCS_BUCKET="temporaryGcsBucket"
MONGO_URL = "spark.mongodb.output.uri"
MONGO_INPUT_URI = "spark.mongodb.input.uri"
MONGO_DATABASE = "database"
MONGO_COLLECTION = "collection"
FORMAT_MONGO = "com.mongodb.spark.sql.DefaultSource"
MONGO_DEFAULT_BATCH_SIZE = 512
MONGO_BATCH_SIZE = "maxBatchSize"
FORMAT_SNOWFLAKE = "net.snowflake.spark.snowflake"
REDSHIFT_TEMPDIR = "tempdir"
REDSHIFT_IAMROLE = "aws_iam_role"
AWS_S3ACCESSKEY = "fs.s3a.access.key"
AWS_S3SECRETKEY = "fs.s3a.secret.key"
AWS_S3ENDPOINT = "fs.s3a.endpoint"
SQL_EXTENSION= "spark.sql.extensions"
CASSANDRA_EXTENSION= "com.datastax.spark.connector.CassandraSparkExtensions"
CASSANDRA_CATALOG= "com.datastax.spark.connector.datasource.CassandraCatalog"
FORMAT_PUBSUBLITE = "pubsublite"

OPTION_DEFAULT = "default"
OPTION_HELP = "help"
OPTION_READ_HELP = "read_help"
OPTION_WRITE_HELP = "write_help"

# At the moment this is just a map of CSV related options but it will be expanded as required for other uses.
SPARK_OPTIONS = {
    CSV_CHARTOESCAPEQUOTEESCAPING:
        {OPTION_HELP: "Sets a single character used for escaping the escape for the quote character. "
                      "The default value is escape character when escape and quote characters are "
                      "different, \\0 otherwise"},
    CSV_COLUMNNAMEOFCORRUPTRECORD:
        {OPTION_READ_HELP: "Allows renaming the new field having malformed "
                           "string created by PERMISSIVE mode"},
    CSV_COMMENT:
        {OPTION_READ_HELP: "Sets a single character used for skipping lines beginning with this "
                           "character. By default it is disabled"},
    CSV_COMPRESSION:
        {OPTION_WRITE_HELP: "Compression codec to use when saving to file. This can be one of the known "
                            "case-insensitive short names (none, bzip2, gzip, lz4, snappy and deflate)"},
    CSV_DATEFORMAT:
        {OPTION_HELP: "Sets the string that indicates a date format. This applies to date type"},
    CSV_EMPTYVALUE:
        {OPTION_HELP: "Sets the string representation of an empty value"},
    CSV_ENCODING:
        {OPTION_READ_HELP: "Decodes the CSV files by the given encoding type",
         OPTION_WRITE_HELP: "Specifies encoding (charset) of saved CSV files"},
    CSV_ENFORCESCHEMA:
        {OPTION_READ_HELP: "If it is set to true, the specified or inferred schema will be "
                           "forcibly applied to datasource files, and headers in CSV files "
                           "will be ignored. If the option is set to false, the schema will "
                           "be validated against all headers in CSV files in the case when "
                           "the header option is set to true. Defaults to True"},
    CSV_ESCAPE:
        {OPTION_HELP: "Sets a single character used for escaping quotes inside an already quoted value"},
    CSV_ESCAPEQUOTES:
        {OPTION_HELP: "A flag indicating whether values containing quotes should always be enclosed "
                      "in quotes. Default is to escape all values containing a quote character"},
    CSV_HEADER:
        {OPTION_DEFAULT: "true",
         OPTION_READ_HELP: "Uses the first line of CSV file as names of columns. Defaults to True",
         OPTION_WRITE_HELP: "Writes the names of columns as the first line. Defaults to True"},
    CSV_IGNORELEADINGWHITESPACE:
        {OPTION_HELP: "A flag indicating whether or not leading whitespaces from "
                      "values being read/written should be skipped"},
    CSV_IGNORETRAILINGWHITESPACE:
        {OPTION_HELP: "A flag indicating whether or not trailing whitespaces from "
                      "values being read/written should be skipped"},
    CSV_INFER_SCHEMA:
        {OPTION_DEFAULT: "true",
         OPTION_READ_HELP: "Infers the input schema automatically from data. It requires one "
                           "extra pass over the data. Defaults to True"},
    CSV_LINESEP:
        {OPTION_HELP: "Defines the line separator that should be used for parsing. "
                      "Defaults to \\r, \\r\\n and \\n for reading and \\n for writing"},
    CSV_LOCALE:
        {OPTION_READ_HELP: "Sets a locale as language tag in IETF BCP 47 format"},
    CSV_MAXCHARSPERCOLUMN:
        {OPTION_READ_HELP: "Defines the maximum number of characters allowed for any "
                           "given value being read. By default, it is -1 meaning unlimited length"},
    CSV_MAXCOLUMNS:
        {OPTION_READ_HELP: "Defines a hard limit of how many columns a record can have"},
    CSV_MODE:
        {OPTION_READ_HELP: "Allows a mode for dealing with corrupt records during parsing. It supports "
                           "the following case-insensitive modes: PERMISSIVE, DROPMALFORMED, FAILFAST"},
    CSV_MULTILINE:
        {OPTION_READ_HELP: "Parse one record, which may span multiple lines, per file"},
    CSV_NANVALUE:
        {OPTION_READ_HELP: "Sets the string representation of a non-number value"},
    CSV_NULLVALUE:
        {OPTION_HELP: "Sets the string representation of a null value"},
    CSV_NEGATIVEINF:
        {OPTION_READ_HELP: "Sets the string representation of a negative infinity value"},
    CSV_POSITIVEINF:
        {OPTION_READ_HELP: "Sets the string representation of a positive infinity value"},
    CSV_QUOTE:
        {OPTION_READ_HELP: "Sets a single character used for escaping quoted values where the separator can "
                           "be part of the value. For reading, if you would like to turn off quotations, "
                           "you need to set not null but an empty string",
         OPTION_WRITE_HELP: "Sets a single character used for escaping quoted values where the separator can "
                            "be part of the value. For writing, if an empty string is set, it uses u0000 "
                            "(null character)"},
    CSV_QUOTEALL:
        {OPTION_WRITE_HELP: "A flag indicating whether all values should always be enclosed in quotes. "
                            "Default is to only escape values containing a quote character"},
    CSV_SAMPLINGRATIO:
        {OPTION_READ_HELP: "Defines fraction of rows used for schema inferring"},
    CSV_SEP:
        {OPTION_HELP: "Sets a separator for each field and value. This separator can be one or more characters"},
    CSV_TIMESTAMPFORMAT:
        {OPTION_HELP: "Sets the string that indicates a timestamp with timezone format"},
    CSV_TIMESTAMPNTZFORMAT:
        {OPTION_HELP: "Sets the string that indicates a timestamp without timezone format"},
    CSV_UNESCAPEDQUOTEHANDLING:
        {OPTION_READ_HELP: "Defines how the CsvParser will handle values with unescaped quotes."
                           "Valid values are: STOP_AT_CLOSING_QUOTE, BACK_TO_DELIMITER, STOP_AT_DELIMITER, SKIP_VALUE, RAISE_ERROR"},
}

# Helper functions for applying SPARK_OPTIONS to templates
def get_csv_input_spark_options(prefix):
    input_options = [
        CSV_CHARTOESCAPEQUOTEESCAPING,
        CSV_COLUMNNAMEOFCORRUPTRECORD,
        CSV_COMMENT,
        CSV_DATEFORMAT,
        CSV_EMPTYVALUE,
        CSV_ENCODING,
        CSV_ENFORCESCHEMA,
        CSV_ESCAPE,
        CSV_HEADER,
        CSV_IGNORELEADINGWHITESPACE,
        CSV_IGNORETRAILINGWHITESPACE,
        CSV_INFER_SCHEMA,
        CSV_LINESEP,
        CSV_LOCALE,
        CSV_MAXCHARSPERCOLUMN,
        CSV_MAXCOLUMNS,
        CSV_MODE,
        CSV_MULTILINE,
        CSV_NANVALUE,
        CSV_NULLVALUE,
        CSV_NEGATIVEINF,
        CSV_POSITIVEINF,
        CSV_QUOTE,
        CSV_SAMPLINGRATIO,
        CSV_SEP,
        CSV_TIMESTAMPFORMAT,
        CSV_TIMESTAMPNTZFORMAT,
        CSV_UNESCAPEDQUOTEHANDLING,
    ]
    spark_options = {(prefix + _).lower(): _ for _ in input_options}
    return spark_options

def get_csv_output_spark_options(prefix):
    output_options = {
        CSV_CHARTOESCAPEQUOTEESCAPING,
        CSV_COMPRESSION,
        CSV_DATEFORMAT,
        CSV_EMPTYVALUE,
        CSV_ENCODING,
        CSV_ESCAPE,
        CSV_ESCAPEQUOTES,
        CSV_HEADER,
        CSV_IGNORELEADINGWHITESPACE,
        CSV_IGNORETRAILINGWHITESPACE,
        CSV_LINESEP,
        CSV_NULLVALUE,
        CSV_QUOTE,
        CSV_QUOTEALL,
        CSV_SEP,
        CSV_TIMESTAMPFORMAT,
        CSV_TIMESTAMPNTZFORMAT,
    }
    spark_options = {(prefix + _).lower(): _ for _ in output_options}
    return spark_options

# Output mode
OUTPUT_MODE_OVERWRITE = "overwrite"
OUTPUT_MODE_APPEND = "append"
OUTPUT_MODE_IGNORE = "ignore"
OUTPUT_MODE_ERRORIFEXISTS = "errorifexists"

# GCS to BigQuery
GCS_BQ_INPUT_LOCATION = "gcs.bigquery.input.location"
GCS_BQ_INPUT_FORMAT = "gcs.bigquery.input.format"
GCS_BQ_OUTPUT_DATASET = "gcs.bigquery.output.dataset"
GCS_BQ_OUTPUT_TABLE = "gcs.bigquery.output.table"
GCS_BQ_OUTPUT_MODE = "gcs.bigquery.output.mode"
GCS_BQ_TEMP_BUCKET = "temporaryGcsBucket"
GCS_BQ_LD_TEMP_BUCKET_NAME = "gcs.bigquery.temp.bucket.name"

# GCS to JDBC
GCS_JDBC_INPUT_LOCATION = "gcs.jdbc.input.location"
GCS_JDBC_INPUT_FORMAT = "gcs.jdbc.input.format"
GCS_JDBC_OUTPUT_TABLE = "gcs.jdbc.output.table"
GCS_JDBC_OUTPUT_MODE = "gcs.jdbc.output.mode"
GCS_JDBC_OUTPUT_URL = "gcs.jdbc.output.url"
GCS_JDBC_OUTPUT_DRIVER = "gcs.jdbc.output.driver"
GCS_JDBC_BATCH_SIZE = "gcs.jdbc.batch.size"
GCS_JDBC_NUMPARTITIONS = "gcs.jdbc.numpartitions"

#GCS to Mongo
GCS_MONGO_INPUT_LOCATION = "gcs.mongo.input.location"
GCS_MONGO_INPUT_FORMAT = "gcs.mongo.input.format"
GCS_MONGO_OUTPUT_URI = "gcs.mongo.output.uri"
GCS_MONGO_OUTPUT_DATABASE = "gcs.mongo.output.database"
GCS_MONGO_OUTPUT_COLLECTION = "gcs.mongo.output.collection"
GCS_MONGO_OUTPUT_MODE = "gcs.mongo.output.mode"
GCS_MONGO_BATCH_SIZE = "gcs.mongo.batch.size"

# Mongo to GCS
MONGO_GCS_OUTPUT_LOCATION = "mongo.gcs.output.location"
MONGO_GCS_OUTPUT_FORMAT = "mongo.gcs.output.format"
MONGO_GCS_OUTPUT_MODE = "mongo.gcs.output.mode"
MONGO_GCS_INPUT_URI = "mongo.gcs.input.uri"
MONGO_GCS_INPUT_DATABASE = "mongo.gcs.input.database"
MONGO_GCS_INPUT_COLLECTION = "mongo.gcs.input.collection"

#Cassandra to BQ
CASSANDRA_TO_BQ_INPUT_TABLE = "cassandratobq.input.table"
CASSANDRA_TO_BQ_INPUT_HOST = "cassandratobq.input.host"
CASSANDRA_TO_BQ_BIGQUERY_LOCATION = "cassandratobq.bigquery.location"
CASSANDRA_TO_BQ_WRITE_MODE = "cassandratobq.output.mode"
CASSANDRA_TO_BQ_TEMP_LOCATION = "cassandratobq.temp.gcs.location"
CASSANDRA_TO_BQ_QUERY = "cassandratobq.input.query"
CASSANDRA_TO_BQ_CATALOG = "cassandratobq.input.catalog.name"
CASSANDRA_TO_BQ_INPUT_KEYSPACE = "cassandratobq.input.keyspace"

# GCS to BigTable
GCS_BT_INPUT_LOCATION = "gcs.bigtable.input.location"
GCS_BT_INPUT_FORMAT = "gcs.bigtable.input.format"
GCS_BT_HBASE_CATALOG_JSON = "gcs.bigtable.hbase.catalog.json"

# BigQuery to GCS
BQ_GCS_INPUT_TABLE = "bigquery.gcs.input.table"
BQ_GCS_OUTPUT_FORMAT = "bigquery.gcs.output.format"
BQ_GCS_OUTPUT_MODE = "bigquery.gcs.output.mode"
BQ_GCS_OUTPUT_LOCATION = "bigquery.gcs.output.location"

# GCS To GCS with transformations
GCS_TO_GCS_INPUT_LOCATION = "gcs.to.gcs.input.location"
GCS_TO_GCS_INPUT_FORMAT = "gcs.to.gcs.input.format"
GCS_TO_GCS_TEMP_VIEW_NAME = "gcs.to.gcs.temp.view.name"
GCS_TO_GCS_SQL_QUERY = "gcs.to.gcs.sql.query"
GCS_TO_GCS_OUTPUT_FORMAT = "gcs.to.gcs.output.format"
GCS_TO_GCS_OUTPUT_MODE = "gcs.to.gcs.output.mode"
GCS_TO_GCS_OUTPUT_PARTITION_COLUMN = "gcs.to.gcs.output.partition.column"
GCS_TO_GCS_OUTPUT_LOCATION = "gcs.to.gcs.output.location"

# Hive to BigQuery
HIVE_BQ_OUTPUT_MODE = "hive.bigquery.output.mode"
HIVE_BQ_LD_TEMP_BUCKET_NAME = "hive.bigquery.temp.bucket.name"
HIVE_BQ_OUTPUT_DATASET = "hive.bigquery.output.dataset"
HIVE_BQ_OUTPUT_TABLE = "hive.bigquery.output.table"
HIVE_BQ_INPUT_DATABASE = "hive.bigquery.input.database"
HIVE_BQ_INPUT_TABLE = "hive.bigquery.input.table"
HIVE_BQ_TEMP_VIEW_NAME = "hive.bigquery.temp.view.name"
HIVE_BQ_SQL_QUERY = "hive.bigquery.sql.query"

# Hive to GCS
HIVE_GCS_INPUT_DATABASE="hive.gcs.input.database"
HIVE_GCS_INPUT_TABLE = "hive.gcs.input.table"
HIVE_GCS_OUTPUT_LOCATION = "hive.gcs.output.location"
HIVE_GCS_OUTPUT_FORMAT = "hive.gcs.output.format"
HIVE_GCS_OUTPUT_MODE = "hive.gcs.output.mode"
HIVE_GCS_TEMP_VIEW_NAME = "hive.gcs.temp.view.name"
HIVE_GCS_SQL_QUERY = "hive.gcs.sql.query"

# Text to BigQuery
TEXT_INPUT_COMPRESSION = "text.bigquery.input.compression"
TEXT_INPUT_DELIMITER = "text.bigquery.input.delimiter"
TEXT_BQ_INPUT_LOCATION = "text.bigquery.input.location"
TEXT_BQ_OUTPUT_DATASET = "text.bigquery.output.dataset"
TEXT_BQ_OUTPUT_TABLE = "text.bigquery.output.table"
TEXT_BQ_OUTPUT_MODE = "text.bigquery.output.mode"
TEXT_BQ_TEMP_BUCKET = "temporaryGcsBucket"
TEXT_BQ_LD_TEMP_BUCKET_NAME = "text.bigquery.temp.bucket.name"

# Hbase to GCS
HBASE_GCS_OUTPUT_LOCATION = "hbase.gcs.output.location"
HBASE_GCS_OUTPUT_FORMAT = "hbase.gcs.output.format"
HBASE_GCS_OUTPUT_MODE = "hbase.gcs.output.mode"
HBASE_GCS_CATALOG_JSON = "hbase.gcs.catalog.json"

# JDBC to JDBC
JDBCTOJDBC_INPUT_URL = "jdbctojdbc.input.url"
JDBCTOJDBC_INPUT_DRIVER = "jdbctojdbc.input.driver"
JDBCTOJDBC_INPUT_TABLE = "jdbctojdbc.input.table"
JDBCTOJDBC_INPUT_FETCHSIZE = "jdbctojdbc.input.fetchsize"
JDBCTOJDBC_INPUT_PARTITIONCOLUMN = "jdbctojdbc.input.partitioncolumn"
JDBCTOJDBC_INPUT_LOWERBOUND = "jdbctojdbc.input.lowerbound"
JDBCTOJDBC_INPUT_UPPERBOUND = "jdbctojdbc.input.upperbound"
JDBCTOJDBC_SESSIONINITSTATEMENT = "jdbctojdbc.input.sessioninitstatement"
JDBCTOJDBC_NUMPARTITIONS = "jdbctojdbc.numpartitions"
JDBCTOJDBC_OUTPUT_URL = "jdbctojdbc.output.url"
JDBCTOJDBC_OUTPUT_DRIVER = "jdbctojdbc.output.driver"
JDBCTOJDBC_OUTPUT_TABLE = "jdbctojdbc.output.table"
JDBCTOJDBC_OUTPUT_CREATE_TABLE_OPTION = "jdbctojdbc.output.create_table.option"
JDBCTOJDBC_OUTPUT_MODE = "jdbctojdbc.output.mode"
JDBCTOJDBC_OUTPUT_BATCH_SIZE = "jdbctojdbc.output.batch.size"
JDBCTOJDBC_TEMP_VIEW_NAME = "jdbctojdbc.temp.view.name"
JDBCTOJDBC_SQL_QUERY = "jdbctojdbc.sql.query"

# JDBC to GCS
JDBCTOGCS_INPUT_URL = "jdbctogcs.input.url"
JDBCTOGCS_INPUT_DRIVER = "jdbctogcs.input.driver"
JDBCTOGCS_INPUT_TABLE = "jdbctogcs.input.table"
JDBCTOGCS_INPUT_SQL_QUERY = "jdbctogcs.input.sql.query"
JDBCTOGCS_INPUT_FETCHSIZE = "jdbctogcs.input.fetchsize"
JDBCTOGCS_INPUT_PARTITIONCOLUMN = "jdbctogcs.input.partitioncolumn"
JDBCTOGCS_INPUT_LOWERBOUND = "jdbctogcs.input.lowerbound"
JDBCTOGCS_INPUT_UPPERBOUND = "jdbctogcs.input.upperbound"
JDBCTOGCS_SESSIONINITSTATEMENT = "jdbctogcs.input.sessioninitstatement"
JDBCTOGCS_NUMPARTITIONS = "jdbctogcs.numpartitions"
JDBCTOGCS_OUTPUT_LOCATION = "jdbctogcs.output.location"
JDBCTOGCS_OUTPUT_FORMAT = "jdbctogcs.output.format"
JDBCTOGCS_OUTPUT_MODE = "jdbctogcs.output.mode"
JDBCTOGCS_OUTPUT_PARTITIONCOLUMN = "jdbctogcs.output.partitioncolumn"
JDBCTOGCS_TEMP_VIEW_NAME = "jdbctogcs.temp.view.name"
JDBCTOGCS_TEMP_SQL_QUERY = "jdbctogcs.temp.sql.query"

# JDBC to BigQuery
JDBC_BQ_INPUT_URL = "jdbc.bigquery.input.url"
JDBC_BQ_INPUT_DRIVER = "jdbc.bigquery.input.driver"
JDBC_BQ_INPUT_TABLE = "jdbc.bigquery.input.table"
JDBC_BQ_INPUT_FETCHSIZE = "jdbc.bigquery.input.fetchsize"
JDBC_BQ_INPUT_PARTITIONCOLUMN = "jdbc.bigquery.input.partitioncolumn"
JDBC_BQ_INPUT_LOWERBOUND = "jdbc.bigquery.input.lowerbound"
JDBC_BQ_INPUT_UPPERBOUND = "jdbc.bigquery.input.upperbound"
JDBC_BQ_SESSIONINITSTATEMENT = "jdbc.bigquery.input.sessioninitstatement"
JDBC_BQ_NUMPARTITIONS = "jdbc.bigquery.numpartitions"
JDBC_BQ_OUTPUT_MODE = "jdbc.bigquery.output.mode"
JDBC_BQ_OUTPUT_DATASET = "jdbc.bigquery.output.dataset"
JDBC_BQ_OUTPUT_TABLE = "jdbc.bigquery.output.table"
JDBC_BQ_OUTPUT_MODE = "jdbc.bigquery.output.mode"
JDBC_BQ_TEMP_BUCKET = "temporaryGcsBucket"
JDBC_BQ_LD_TEMP_BUCKET_NAME = "jdbc.bigquery.temp.bucket.name"

#REDSHIFT to GCS
REDSHIFTTOGCS_INPUT_URL = "redshifttogcs.input.url"
REDSHIFTTOGCS_S3_TEMPDIR = "redshifttogcs.s3.tempdir"
REDSHIFTTOGCS_INPUT_TABLE = "redshifttogcs.input.table"
REDSHIFTTOGCS_IAM_ROLEARN = "redshifttogcs.iam.rolearn"
REDSHIFTTOGCS_S3_ACCESSKEY = "redshifttogcs.s3.accesskey"
REDSHIFTTOGCS_S3_SECRETKEY = "redshifttogcs.s3.secretkey"
REDSHIFTTOGCS_OUTPUT_LOCATION = "redshifttogcs.output.location"
REDSHIFTTOGCS_OUTPUT_FORMAT = "redshifttogcs.output.format"
REDSHIFTTOGCS_OUTPUT_MODE = "redshifttogcs.output.mode"
REDSHIFTTOGCS_OUTPUT_PARTITIONCOLUMN = "redshifttogcs.output.partitioncolumn"

# Snowflake To GCS
SNOWFLAKE_TO_GCS_SF_URL = "snowflake.to.gcs.sf.url"
SNOWFLAKE_TO_GCS_SF_USER = "snowflake.to.gcs.sf.user"
SNOWFLAKE_TO_GCS_SF_PASSWORD = "snowflake.to.gcs.sf.password"
SNOWFLAKE_TO_GCS_SF_DATABASE = "snowflake.to.gcs.sf.database"
SNOWFLAKE_TO_GCS_SF_SCHEMA = "snowflake.to.gcs.sf.schema"
SNOWFLAKE_TO_GCS_SF_WAREHOUSE = "snowflake.to.gcs.sf.warehouse"
SNOWFLAKE_TO_GCS_SF_AUTOPUSHDOWN = "snowflake.to.gcs.sf.autopushdown"
SNOWFLAKE_TO_GCS_SF_TABLE = "snowflake.to.gcs.sf.table"
SNOWFLAKE_TO_GCS_SF_QUERY = "snowflake.to.gcs.sf.query"
SNOWFLAKE_TO_GCS_OUTPUT_LOCATION = "snowflake.to.gcs.output.location"
SNOWFLAKE_TO_GCS_OUTPUT_MODE = "snowflake.to.gcs.output.mode"
SNOWFLAKE_TO_GCS_OUTPUT_FORMAT = "snowflake.to.gcs.output.format"
SNOWFLAKE_TO_GCS_PARTITION_COLUMN = "snowflake.to.gcs.partition.column"

# Cassandra To GCS
CASSANDRA_TO_GCS_INPUT_KEYSPACE = "cassandratogcs.input.keyspace"
CASSANDRA_TO_GCS_INPUT_TABLE = "cassandratogcs.input.table"
CASSANDRA_TO_GCS_INPUT_HOST = "cassandratogcs.input.host"
CASSANDRA_TO_GCS_OUTPUT_FORMAT = "cassandratogcs.output.format"
CASSANDRA_TO_GCS_OUTPUT_PATH = "cassandratogcs.output.path"
CASSANDRA_TO_GCS_OUTPUT_SAVEMODE = "cassandratogcs.output.savemode"
CASSANDRA_TO_GCS_CATALOG = "cassandratogcs.input.catalog.name"
CASSANDRA_TO_GCS_QUERY = "cassandratogcs.input.query"

# Hive DDL Extractor Util
HIVE_DDL_EXTRACTOR_INPUT_DATABASE = "hive.ddl.extractor.input.database"
HIVE_DDL_EXTRACTOR_OUTPUT_GCS_PATH = "hive.ddl.extractor.output.path"
HIVE_DDL_CONSIDER_SPARK_TABLES = "hive.ddl.consider.spark.tables"
HIVE_DDL_TRANSLATION_DISPOSITION = "hive.ddl.translation.disposition"

# AWS S3 To BigQuery
S3_BQ_INPUT_LOCATION = "s3.bq.input.location"
S3_BQ_INPUT_FORMAT = "s3.bq.input.format"
S3_BQ_ACCESS_KEY = "s3.bq.access.key"
S3_BQ_SECRET_KEY = "s3.bq.secret.key"
S3_BQ_OUTPUT_DATASET_NAME = "s3.bq.output.dataset.name"
S3_BQ_OUTPUT_TABLE_NAME = "s3.bq.output.table.name"
S3_BQ_TEMP_BUCKET_NAME = "s3.bq.temp.bucket.name"
S3_BQ_OUTPUT_MODE = "s3.bq.output.mode"
S3_BQ_ENDPOINT_VALUE = "s3.amazonaws.com"


#Kafka To Bq
KAFKA_BQ_CHECKPOINT_LOCATION='kafka.to.bq.checkpoint.location'
KAFKA_BOOTSTRAP_SERVERS='kafka.to.bq.bootstrap.servers'
KAFKA_BQ_TOPIC='kafka.to.bq.topic'
KAFKA_BQ_STARTING_OFFSET='kafka.to.bq.starting.offset'
KAFKA_BQ_DATASET='kafka.to.bq.dataset'
KAFKA_BQ_TABLE_NAME='kafka.to.bq.table'
KAFKA_BQ_TEMP_BUCKET_NAME='kafka.to.bq.temp.bucket.name'
KAFKA_BQ_TERMINATION_TIMEOUT='kafka.to.bq.termination.timeout'
KAFKA_INPUT_FORMAT='kafka'
KAFKA_BQ_OUTPUT_MODE='kafka.to.bq.output.mode'

KAFKA_INPUT_FORMAT='kafka'

#Kafka To GCS
KAFKA_GCS_BOOTSTRAP_SERVERS='kafka.gcs.bootstrap.servers'
KAFKA_GCS_OUTPUT_LOCATION='kafka.gcs.output.location.gcs.path'
KAFKA_TOPIC='kafka.gcs.topic'
KAFKA_GCS_OUTPUT_FORMAT='kafka.gcs.output.format'
KAFKA_GCS_OUPUT_MODE='kafka.gcs.output.mode'
KAFKA_GCS_TERMINATION_TIMEOUT='kafka.gcs.termination.timeout'
KAFKA_STARTING_OFFSET='kafka.gcs.starting.offset'
KAFKA_GCS_CHECKPOINT_LOCATION='kafka.gcs.checkpoint.location'

#Pubsublite To GCS
PUBSUBLITE_TO_GCS_INPUT_SUBSCRIPTION_URL = "pubsublite.to.gcs.input.subscription.url"
PUBSUBLITE_TO_GCS_WRITE_MODE = "pubsublite.to.gcs.write.mode"
PUBSUBLITE_TO_GCS_OUTPUT_LOCATION = "pubsublite.to.gcs.output.location"
PUBSUBLITE_CHECKPOINT_LOCATION = "pubsublite.to.gcs.checkpoint.location"
PUBSUBLITE_TO_GCS_OUTPUT_FORMAT = "pubsublite.to.gcs.output.format"
PUBSUBLITE_TO_GCS_TIMEOUT = "pubsublite.to.gcs.timeout"
PUBSUBLITE_TO_GCS_PROCESSING_TIME = "pubsublite.to.gcs.processing.time"

# Azure Storage to BigQuery
AZ_BLOB_BQ_INPUT_LOCATION = "azure.blob.bigquery.input.location"
AZ_BLOB_BQ_INPUT_FORMAT = "azure.blob.bigquery.input.format"
AZ_BLOB_BQ_OUTPUT_DATASET = "azure.blob.bigquery.output.dataset"
AZ_BLOB_BQ_OUTPUT_TABLE = "azure.blob.bigquery.output.table"
AZ_BLOB_BQ_OUTPUT_MODE = "azure.blob.bigquery.output.mode"
AZ_BLOB_BQ_TEMP_BUCKET = "temporaryGcsBucket"
AZ_BLOB_BQ_LD_TEMP_BUCKET_NAME = "azure.blob.bigquery.temp.bucket.name"
AZ_BLOB_STORAGE_ACCOUNT = "azure.blob.storage.account"
AZ_BLOB_CONTAINER_NAME = "azure.blob.container.name"
AZ_BLOB_SAS_TOKEN = "azure.blob.sas.token"



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.azure.azure_blob_storage_to_bigquery import AzureBlobStorageToBigQueryTemplate
import dataproc_templates.util.template_constants as constants


class TestAzureBlobStorageToBigQueryTemplate:
    """
    Test suite for AzureBlobStorageToBigQueryTemplate
    """

    def test_parse_args(self):
        """Tests AzureBlobStorageToBigQueryTemplate.parse_args()"""

        azure_blob_storage_to_bigquery_template = AzureBlobStorageToBigQueryTemplate()
        parsed_args = azure_blob_storage_to_bigquery_template.parse_args([
            "--azure.blob.bigquery.input.format=parquet",
            "--azure.blob.bigquery.output.mode=append",
            "--azure.blob.bigquery.input.location=gs://test",
            "--azure.blob.bigquery.output.dataset=dataset",
            "--azure.blob.bigquery.output.table=table",
            "--azure.blob.bigquery.temp.bucket.name=bucket",
            "--azure.blob.storage.account=test_storage_account",
            "--azure.blob.container.name=test_container",
            "--azure.blob.sas.token=test_sas_token"])

        assert parsed_args["azure.blob.bigquery.input.format"] == "parquet"
        assert parsed_args["azure.blob.bigquery.output.mode"] == "append"
        assert parsed_args["azure.blob.bigquery.input.location"] == "gs://test"
        assert parsed_args["azure.blob.bigquery.output.dataset"] == "dataset"
        assert parsed_args["azure.blob.bigquery.output.table"] == "table"
        assert parsed_args["azure.blob.bigquery.temp.bucket.name"] == "bucket"
        assert parsed_args["azure.blob.storage.account"] == "test_storage_account"
        assert parsed_args["azure.blob.container.name"] == "test_container"
        assert parsed_args["azure.blob.sas.token"] == "test_sas_token"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_parquet(self, mock_spark_session):
        """Tests GCSToBigqueryTemplate runs with parquet format"""

        azure_blob_storage_to_bigquery_template = AzureBlobStorageToBigQueryTemplate()
        mock_parsed_args = azure_blob_storage_to_bigquery_template.parse_args([
            "--azure.blob.bigquery.input.format=parquet",
            "--azure.blob.bigquery.output.mode=append",
            "--azure.blob.bigquery.input.location=gs://test",
            "--azure.blob.bigquery.output.dataset=dataset",
            "--azure.blob.bigquery.output.table=table",
            "--azure.blob.bigquery.temp.bucket.name=bucket",
            "--azure.blob.storage.account=test_storage_account",
            "--azure.blob.container.name=test_container",
            "--azure.blob.sas.token=test_sas_token"])
        mock_spark_session.read.parquet.return_value = mock_spark_session.dataframe.DataFrame
        azure_blob_storage_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.parquet.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint
import sys

from pyspark.sql import SparkSession, DataFrame, DataFrameWriter

from dataproc_templates import BaseTemplate
from dataproc_templates.util.argument_parsing import add_spark_options
from dataproc_templates.util.dataframe_writer_wrappers import persist_dataframe_to_cloud_storage
import dataproc_templates.util.template_constants as constants

from pyspark import SparkConf, SparkContext
from pyspark.sql import SparkSession


__all__ = ['CassandraToGCSTemplate']


class CassandraToGCSTemplate(BaseTemplate):
   """
   Dataproc template implementing exports from CASSANDRA to GCS
   """

   @staticmethod
   def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
       parser: argparse.ArgumentParser = argparse.ArgumentParser()

       parser.add_argument(
           f'--{constants.CASSANDRA_TO_GCS_INPUT_HOST}',
           dest=constants.CASSANDRA_TO_GCS_INPUT_HOST,
           required=True,
           help='CASSANDRA Cloud Storage Input Host IP'
       )
       parser.add_argument(
           f'--{constants.CASSANDRA_TO_GCS_OUTPUT_FORMAT}',
           dest=constants.CASSANDRA_TO_GCS_OUTPUT_FORMAT,
           required=True,
           help='Output file format (one of: avro,parquet,csv,json)',
           choices=[
               constants.FORMAT_AVRO,
               constants.FORMAT_PRQT,
               constants.FORMAT_CSV,
               constants.FORMAT_JSON
           ]
       )
       parser.add_argument(
           f'--{constants.CASSANDRA_TO_GCS_OUTPUT_PATH}',
           dest=constants.CASSANDRA_TO_GCS_OUTPUT_PATH,
           required=True,
           help='Cloud Storage location for output files'
       )
       parser.add_argument(
           f'--{constants.CASSANDRA_TO_GCS_OUTPUT_SAVEMODE}',
           dest=constants.CASSANDRA_TO_GCS_OUTPUT_SAVEMODE,
           required=False,
           default=constants.OUTPUT_MODE_APPEND,
           help=(
               'Output write mode '
               '(one of: append,overwrite,ignore,errorifexists) '
               '(Defaults to append)'
           ),
           choices=[
               constants.OUTPUT_MODE_OVERWRITE,
               constants.OUTPUT_MODE_APPEND,
               constants.OUTPUT_MODE_IGNORE,
               constants.OUTPUT_MODE_ERRORIFEXISTS
           ]
       )
       parser.add_argument(
            f'--{constants.CASSANDRA_TO_GCS_CATALOG}',
            dest=constants.CASSANDRA_TO_GCS_CATALOG,
            required=False,
            default="casscon",
            help='To provide a name for connection between Cassandra and GCS'
       )
       parser.add_argument(
            f'--{constants.CASSANDRA_TO_GCS_QUERY}',
            dest=constants.CASSANDRA_TO_GCS_QUERY,
            required=False,
            help='Optional query for selective exports'
       )

       parser.add_argument(
            f'--{constants.CASSANDRA_TO_GCS_INPUT_KEYSPACE}',
            dest=constants.CASSANDRA_TO_GCS_INPUT_KEYSPACE,
            required=(constants.CASSANDRA_TO_GCS_QUERY is None),
            help='CASSANDRA Cloud Storage Input Keyspace'
       )
       parser.add_argument(
           f'--{constants.CASSANDRA_TO_GCS_INPUT_TABLE}',
           dest=constants.CASSANDRA_TO_GCS_INPUT_TABLE,
           required=(constants.CASSANDRA_TO_GCS_QUERY is None),
           help='CASSANDRA Cloud Storage Input Table'
       )
       add_spark_options(parser, constants.get_csv_output_spark_options("cassandra.gcs.output."))

       known_args: argparse.Namespace
       known_args, _ = parser.parse_known_args(args)
       if (not getattr(known_args, constants.CASSANDRA_TO_GCS_QUERY)
            and (not getattr(known_args, constants.CASSANDRA_TO_GCS_INPUT_KEYSPACE)
            or not getattr(known_args, constants.CASSANDRA_TO_GCS_INPUT_TABLE))):

            sys.exit("ArgumentParser Error: Either of cassandratogcs.input.keyspace and cassandratogcs.input.table "
                        + "OR cassandratogcs.input.query needs to be provided as argument to read data from Cassandra")

       elif (getattr(known_args, constants.CASSANDRA_TO_GCS_QUERY)
            and (getattr(known_args, constants.CASSANDRA_TO_GCS_INPUT_KEYSPACE)
            or getattr(known_args, constants.CASSANDRA_TO_GCS_INPUT_TABLE))):

            sys.exit("ArgumentParser Error: Both cassandratogcs.input.keyspace and cassandratogcs.input.table "
                        + "AND cassandratogcs.input.query cannot be provided as arguments at the same time.")


       return vars(known_args)

   def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

       logger: Logger = self.get_logger(spark=spark)

       # Arguments
       input_host: str = args[constants.CASSANDRA_TO_GCS_INPUT_HOST]
       input_keyspace: str = args[constants.CASSANDRA_TO_GCS_INPUT_KEYSPACE]
       input_table: str = args[constants.CASSANDRA_TO_GCS_INPUT_TABLE]
       output_format: str = args[constants.CASSANDRA_TO_GCS_OUTPUT_FORMAT]
       output_location: str = args[constants.CASSANDRA_TO_GCS_OUTPUT_PATH]
       output_mode: str = args[constants.CASSANDRA_TO_GCS_OUTPUT_SAVEMODE]
       catalog: str = args[constants.CASSANDRA_TO_GCS_CATALOG]
       query: str = args[constants.CASSANDRA_TO_GCS_QUERY]

       logger.info(
           "Starting CASSANDRA to Cloud Storage spark job with parameters:\n"
           f"{pprint.pformat(args)}"
       )

       # Set configuration to connect to Cassandra by overwriting the spark session
       spark = (
        SparkSession
        .builder
        .appName("CassandraToGCS")
        .config(constants.SQL_EXTENSION, constants.CASSANDRA_EXTENSION)
        .config(f"spark.sql.catalog.{catalog}", constants.CASSANDRA_CATALOG)
        .config(f"spark.sql.catalog.{catalog}.spark.cassandra.connection.host",input_host)
        .getOrCreate())

       # Read
       if(not query):
           input_data = spark.read.table(f"{catalog}.{input_keyspace}.{input_table}")
       else:
           input_data = spark.sql(query)

       # Write
       writer: DataFrameWriter = input_data.write.mode(output_mode)
       persist_dataframe_to_cloud_storage(writer, args, output_location, output_format, "cassandratogcs.output.")



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.hive.hive_to_gcs import HiveToGCSTemplate
import dataproc_templates.util.template_constants as constants


class TestHiveToGCSTemplate:
    """
    Test suite for HiveToGCSTemplate
    """

    def test_parse_args(self):
        """Tests HiveToGCSTemplate.parse_args()"""

        hive_to_gcs_template = HiveToGCSTemplate()
        parsed_args = hive_to_gcs_template.parse_args(
            ["--hive.gcs.input.database=database",
             "--hive.gcs.input.table=table",
             "--hive.gcs.output.location=gs://test",
             "--hive.gcs.output.format=parquet",
             "--hive.gcs.output.mode=overwrite"])

        assert parsed_args["hive.gcs.input.database"] == "database"
        assert parsed_args["hive.gcs.input.table"] == "table"
        assert parsed_args["hive.gcs.output.location"] == "gs://test"
        assert parsed_args["hive.gcs.output.format"] == "parquet"
        assert parsed_args["hive.gcs.output.mode"] == "overwrite"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_parquet(self, mock_spark_session):
        """Tests HiveToGCSTemplate runs for parquet format output"""

        hive_to_gcs_template = HiveToGCSTemplate()
        mock_parsed_args = hive_to_gcs_template.parse_args(
            ["--hive.gcs.input.database=database",
             "--hive.gcs.input.table=table",
             "--hive.gcs.output.location=gs://test",
             "--hive.gcs.output.format=parquet",
             "--hive.gcs.output.mode=overwrite"])
        mock_spark_session.table.return_value = mock_spark_session.dataframe.DataFrame
        hive_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.table.assert_called_once_with("database.table")
        mock_spark_session.dataframe.DataFrame.write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.dataframe.DataFrame.write \
            .mode() \
            .parquet.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_avro(self, mock_spark_session):
        """Tests HiveToGCSTemplate runs for avro format output"""

        hive_to_gcs_template = HiveToGCSTemplate()
        mock_parsed_args = hive_to_gcs_template.parse_args(
            ["--hive.gcs.input.database=database",
             "--hive.gcs.input.table=table",
             "--hive.gcs.output.location=gs://test",
             "--hive.gcs.output.format=avro",
             "--hive.gcs.output.mode=append"])
        mock_spark_session.table.return_value = mock_spark_session.dataframe.DataFrame
        hive_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.table.assert_called_once_with("database.table")
        mock_spark_session.dataframe.DataFrame.write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write \
            .mode() \
            .format.assert_called_once_with(constants.FORMAT_AVRO)
        mock_spark_session.dataframe.DataFrame.write \
            .mode() \
            .format() \
            .save.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_csv(self, mock_spark_session):
        """Tests HiveToGCSTemplate runs for csv format output"""

        hive_to_gcs_template = HiveToGCSTemplate()
        mock_parsed_args = hive_to_gcs_template.parse_args(
            ["--hive.gcs.input.database=database",
             "--hive.gcs.input.table=table",
             "--hive.gcs.output.location=gs://test",
             "--hive.gcs.output.format=csv",
             "--hive.gcs.output.mode=ignore"])
        mock_spark_session.table.return_value = mock_spark_session.dataframe.DataFrame
        hive_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.table.assert_called_once_with("database.table")
        mock_spark_session.dataframe.DataFrame.write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write \
            .mode() \
            .options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})
        mock_spark_session.dataframe.DataFrame.write \
            .mode() \
            .options() \
            .csv.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_json(self, mock_spark_session):
        """Tests HiveToGCSTemplate runs with json format output"""

        hive_to_gcs_template = HiveToGCSTemplate()
        mock_parsed_args = hive_to_gcs_template.parse_args(
            ["--hive.gcs.input.database=database",
             "--hive.gcs.input.table=table",
             "--hive.gcs.output.location=gs://test",
             "--hive.gcs.output.format=json",
             "--hive.gcs.output.mode=errorifexists"])
        mock_spark_session.table.return_value = mock_spark_session.dataframe.DataFrame
        hive_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.table.assert_called_once_with("database.table")
        mock_spark_session.dataframe.DataFrame.write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_ERRORIFEXISTS)
        mock_spark_session.dataframe.DataFrame.write \
            .mode() \
            .json.assert_called_once_with("gs://test")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from typing import Dict, Sequence, Optional, Any
from abc import ABC as AbstractClass, abstractmethod
from logging import Logger

from pyspark.sql import SparkSession

__all__ = ['BaseTemplate']


class BaseTemplate(AbstractClass):
    """Base class for all Dataproc Templates"""

    def get_logger(self, spark: SparkSession) -> Logger:
        """
        Convenience method to get the Spark logger from a SparkSession

        Args:
            spark (SparkSession): The initialized SparkSession object

        Returns:
            Logger: The Spark logger
        """

        log_4j_logger = spark.sparkContext._jvm.org.apache.log4j  # pylint: disable=protected-access
        return log_4j_logger.LogManager.getLogger(__name__)

    @classmethod
    def build(cls) -> BaseTemplate:
        """
        Factory method for building an instance of this template class.
        """

        return cls()

    @staticmethod
    @abstractmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        """
        Parses this template's arguments, returning them as a dictionary.

        Implementations of this method should ignore unknown arguments.

        Args:
            args (Optional[Sequence[str]]): The template arguments.
                By default, command line arguments are used.
        """

    @abstractmethod
    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:
        """
        Runs this template
        """



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark.sql

from dataproc_templates.s3.s3_to_bigquery import S3ToBigQueryTemplate
import dataproc_templates.util.template_constants as constants


class TestS3ToBigQueryTemplate:
    """
    Test suite for S3ToBigQueryTemplate
    """

    def test_parse_args1(self):
        """Tests S3ToBigQueryTemplate.parse_args()"""

        s3_to_bigquery_template = S3ToBigQueryTemplate()
        parsed_args = s3_to_bigquery_template.parse_args(
            ["--s3.bq.input.location=s3a://bucket/file",
             "--s3.bq.access.key=SomeAccessKey",
             "--s3.bq.secret.key=SomeSecretKey",
             "--s3.bq.input.format=csv",
             "--s3.bq.output.dataset.name=xyz",
             "--s3.bq.output.table.name=lmn",
             "--s3.bq.temp.bucket.name=test",
             "--s3.bq.output.mode=append"
             ])

        assert parsed_args["s3.bq.input.location"] == "s3a://bucket/file"
        assert parsed_args["s3.bq.access.key"] == "SomeAccessKey"
        assert parsed_args["s3.bq.secret.key"] == "SomeSecretKey"
        assert parsed_args["s3.bq.input.format"] == "csv"
        assert parsed_args["s3.bq.output.dataset.name"] == "xyz"
        assert parsed_args["s3.bq.output.table.name"] == "lmn"
        assert parsed_args["s3.bq.temp.bucket.name"] == "test"
        assert parsed_args["s3.bq.output.mode"] == "append"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args2(self, mock_spark_session):
        """Tests S3ToBigQueryTemplate reads parquet"""

        s3_to_bigquery_template = S3ToBigQueryTemplate()

        mock_parsed_args = s3_to_bigquery_template.parse_args(
            ["--s3.bq.input.location=s3a://bucket/file",
             "--s3.bq.access.key=SomeAccessKey",
             "--s3.bq.secret.key=SomeSecretKey",
             "--s3.bq.input.format=parquet",
             "--s3.bq.output.dataset.name=xyz",
             "--s3.bq.output.table.name=lmn",
             "--s3.bq.temp.bucket.name=test",
             "--s3.bq.output.mode=append"
             ])
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ACCESSKEY, "SomeAccessKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3SECRETKEY, "SomeSecretKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ENDPOINT, constants.S3_BQ_ENDPOINT_VALUE)

        mock_spark_session.read \
            .parquet \
            .return_value = mock_spark_session.dataframe.DataFrame
        s3_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read \
            .parquet \
            .assert_called_with("s3a://bucket/file")

        mock_spark_session.dataframe.DataFrame.write \
            .format \
            .assert_called_once_with(constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option \
            .assert_called_once_with(constants.TABLE, "xyz.lmn")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option \
            .assert_called_once_with(constants.TEMP_GCS_BUCKET, "test")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode \
            .assert_called_once_with(constants.OUTPUT_MODE_APPEND)

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args3(self, mock_spark_session):
        """Tests S3ToBigQueryTemplate reads avro"""

        s3_to_bigquery_template = S3ToBigQueryTemplate()

        mock_parsed_args = s3_to_bigquery_template.parse_args(
            ["--s3.bq.input.location=s3a://bucket/file",
             "--s3.bq.access.key=SomeAccessKey",
             "--s3.bq.secret.key=SomeSecretKey",
             "--s3.bq.input.format=avro",
             "--s3.bq.output.dataset.name=xyz",
             "--s3.bq.output.table.name=lmn",
             "--s3.bq.temp.bucket.name=test",
             "--s3.bq.output.mode=overwrite"
             ])
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ACCESSKEY, "SomeAccessKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3SECRETKEY, "SomeSecretKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ENDPOINT, constants.S3_BQ_ENDPOINT_VALUE)

        mock_spark_session.read \
            .format() \
            .load \
            .return_value = mock_spark_session.dataframe.DataFrame
        s3_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read \
            .format \
            .assert_called_with(constants.FORMAT_AVRO_EXTD)
        mock_spark_session.read \
            .format() \
            .load \
            .assert_called_with("s3a://bucket/file")

        mock_spark_session.dataframe.DataFrame.write \
            .format \
            .assert_called_once_with(constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option \
            .assert_called_once_with(constants.TABLE, "xyz.lmn")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option \
            .assert_called_once_with(constants.TEMP_GCS_BUCKET, "test")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode \
            .assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args4(self, mock_spark_session):
        """Tests S3ToBigQueryTemplate reads csv"""

        s3_to_bigquery_template = S3ToBigQueryTemplate()

        mock_parsed_args = s3_to_bigquery_template.parse_args(
            ["--s3.bq.input.location=s3a://bucket/file",
             "--s3.bq.access.key=SomeAccessKey",
             "--s3.bq.secret.key=SomeSecretKey",
             "--s3.bq.input.format=csv",
             "--s3.bq.output.dataset.name=xyz",
             "--s3.bq.output.table.name=lmn",
             "--s3.bq.temp.bucket.name=test",
             "--s3.bq.output.mode=ignore"
             ])
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ACCESSKEY, "SomeAccessKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3SECRETKEY, "SomeSecretKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ENDPOINT, constants.S3_BQ_ENDPOINT_VALUE)

        mock_spark_session.read \
            .format() \
            .option() \
            .option() \
            .load \
            .return_value = mock_spark_session.dataframe.DataFrame
        s3_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read \
            .format \
            .assert_called_with(constants.FORMAT_CSV)
        mock_spark_session.read \
            .format() \
            .option \
            .assert_called_with('header',True)
        mock_spark_session.read \
            .format() \
            .option() \
            .option \
            .assert_called_with('inferSchema',True)
        mock_spark_session.read \
            .format() \
            .option() \
            .option() \
            .load \
            .assert_called_with("s3a://bucket/file")

        mock_spark_session.dataframe.DataFrame.write \
            .format \
            .assert_called_once_with(constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option \
            .assert_called_once_with(constants.TABLE, "xyz.lmn")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option \
            .assert_called_once_with(constants.TEMP_GCS_BUCKET, "test")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode \
            .assert_called_once_with(constants.OUTPUT_MODE_IGNORE)

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args5(self, mock_spark_session):
        """Tests S3ToBigQueryTemplate reads json"""

        s3_to_bigquery_template = S3ToBigQueryTemplate()

        mock_parsed_args = s3_to_bigquery_template.parse_args(
            ["--s3.bq.input.location=s3a://bucket/file",
             "--s3.bq.access.key=SomeAccessKey",
             "--s3.bq.secret.key=SomeSecretKey",
             "--s3.bq.input.format=json",
             "--s3.bq.output.dataset.name=xyz",
             "--s3.bq.output.table.name=lmn",
             "--s3.bq.temp.bucket.name=test",
             "--s3.bq.output.mode=errorifexists"
             ])
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ACCESSKEY, "SomeAccessKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3SECRETKEY, "SomeSecretKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ENDPOINT, constants.S3_BQ_ENDPOINT_VALUE)

        mock_spark_session.read \
            .json \
            .return_value = mock_spark_session.dataframe.DataFrame
        s3_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read \
            .json \
            .assert_called_with("s3a://bucket/file")

        mock_spark_session.dataframe.DataFrame.write \
            .format \
            .assert_called_once_with(constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option \
            .assert_called_once_with(constants.TABLE, "xyz.lmn")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option \
            .assert_called_once_with(constants.TEMP_GCS_BUCKET, "test")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode \
            .assert_called_once_with(constants.OUTPUT_MODE_ERRORIFEXISTS)

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args6(self, mock_spark_session):
        """Tests S3ToBigQueryTemplate write mode defaults to append"""

        s3_to_bigquery_template = S3ToBigQueryTemplate()

        mock_parsed_args = s3_to_bigquery_template.parse_args(
            ["--s3.bq.input.location=s3a://bucket/file",
             "--s3.bq.access.key=SomeAccessKey",
             "--s3.bq.secret.key=SomeSecretKey",
             "--s3.bq.input.format=json",
             "--s3.bq.output.dataset.name=xyz",
             "--s3.bq.output.table.name=lmn",
             "--s3.bq.temp.bucket.name=test"
             ])
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ACCESSKEY, "SomeAccessKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3SECRETKEY, "SomeSecretKey")
        mock_spark_session._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ENDPOINT, constants.S3_BQ_ENDPOINT_VALUE)

        mock_spark_session.read \
            .json \
            .return_value = mock_spark_session.dataframe.DataFrame
        s3_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read \
            .json \
            .assert_called_with("s3a://bucket/file")

        mock_spark_session.dataframe.DataFrame.write \
            .format \
            .assert_called_once_with(constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option \
            .assert_called_once_with(constants.TABLE, "xyz.lmn")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option \
            .assert_called_once_with(constants.TEMP_GCS_BUCKET, "test")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode \
            .assert_called_once_with(constants.OUTPUT_MODE_APPEND)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint

from pyspark.sql import SparkSession, DataFrame, DataFrameWriter

from dataproc_templates import BaseTemplate
from dataproc_templates.util.argument_parsing import add_spark_options
from dataproc_templates.util.dataframe_writer_wrappers import persist_dataframe_to_cloud_storage
import dataproc_templates.util.template_constants as constants


__all__ = ['RedshiftToGCSTemplate']


class RedshiftToGCSTemplate(BaseTemplate):
    """
    Dataproc template implementing loads from REDSHIFT into Cloud Storage
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_INPUT_URL}',
            dest=constants.REDSHIFTTOGCS_INPUT_URL,
            required=True,
            help='REDSHIFT input URL'
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_INPUT_TABLE}',
            dest=constants.REDSHIFTTOGCS_INPUT_TABLE,
            required=True,
            help='REDSHIFT input table name'
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_S3_TEMPDIR}',
            dest=constants.REDSHIFTTOGCS_S3_TEMPDIR,
            required=True,
            help='REDSHIFT S3 temporary bucket location s3a://bucket/path'
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_IAM_ROLEARN}',
            dest=constants.REDSHIFTTOGCS_IAM_ROLEARN,
            required=True,
            help='REDSHIFT IAM Role with S3 Access'
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_S3_ACCESSKEY}',
            dest=constants.REDSHIFTTOGCS_S3_ACCESSKEY,
            required=True,
            default="",
            help='AWS Access Keys which allow access to S3'
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_S3_SECRETKEY}',
            dest=constants.REDSHIFTTOGCS_S3_SECRETKEY,
            required=True,
            default="",
            help='AWS Secret Keys which allow access to S3'
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_OUTPUT_LOCATION}',
            dest=constants.REDSHIFTTOGCS_OUTPUT_LOCATION,
            required=True,
            help='Cloud Storage location for output files'
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_OUTPUT_FORMAT}',
            dest=constants.REDSHIFTTOGCS_OUTPUT_FORMAT,
            required=True,
            help='Output file format (one of: avro,parquet,csv,json)',
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON
            ]
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_OUTPUT_MODE}',
            dest=constants.REDSHIFTTOGCS_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        parser.add_argument(
            f'--{constants.REDSHIFTTOGCS_OUTPUT_PARTITIONCOLUMN}',
            dest=constants.REDSHIFTTOGCS_OUTPUT_PARTITIONCOLUMN,
            required=False,
            default="",
            help='Cloud Storage partition column name'
        )
        add_spark_options(parser, constants.get_csv_output_spark_options("redshift.gcs.output."))

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_redshift_url: str = args[constants.REDSHIFTTOGCS_INPUT_URL]
        input_redshift_tempdir: str = args[constants.REDSHIFTTOGCS_S3_TEMPDIR]
        input_redshift_table: str = args[constants.REDSHIFTTOGCS_INPUT_TABLE]
        input_redshift_iam_role: str = args[constants.REDSHIFTTOGCS_IAM_ROLEARN]
        input_redshift_accesskey: str = args[constants.REDSHIFTTOGCS_S3_ACCESSKEY]
        input_redshift_secretkey: str = args[constants.REDSHIFTTOGCS_S3_SECRETKEY]

        output_location: str = args[constants.REDSHIFTTOGCS_OUTPUT_LOCATION]
        output_format: str = args[constants.REDSHIFTTOGCS_OUTPUT_FORMAT]
        output_mode: str = args[constants.REDSHIFTTOGCS_OUTPUT_MODE]
        output_partitioncolumn: str = args[constants.REDSHIFTTOGCS_OUTPUT_PARTITIONCOLUMN]

        logger.info(
            "Starting REDSHIFT to Cloud Storage Spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        # Read
        input_data: DataFrame

        spark._jsc.hadoopConfiguration().set(constants.AWS_S3ACCESSKEY, input_redshift_accesskey)
        spark._jsc.hadoopConfiguration().set(constants.AWS_S3SECRETKEY, input_redshift_secretkey)

        input_data=spark.read \
                .format(constants.FORMAT_REDSHIFT) \
                .option(constants.JDBC_URL, input_redshift_url) \
                .option(constants.JDBC_TABLE, input_redshift_table) \
                .option(constants.REDSHIFT_TEMPDIR, input_redshift_tempdir) \
                .option(constants.REDSHIFT_IAMROLE, input_redshift_iam_role) \
                .load()

        # Write
        if (output_partitioncolumn != ""):
            writer: DataFrameWriter = input_data.write.mode(output_mode).partitionBy(output_partitioncolumn)
        else:
            writer: DataFrameWriter = input_data.write.mode(output_mode)

        persist_dataframe_to_cloud_storage(writer, args, output_location, output_format, "redshift.gcs.output.")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataproc_templates import TemplateName

import google.auth
from google.api_core import client_info as http_client_info
from google.cloud import bigquery

from pyspark.sql import SparkSession

def track_template_invocation(spark: SparkSession, template_name: TemplateName) -> None:
    """
    Track template invocation

    Args:
        spark (SparkSession): SparkSession
        template_name (TemplateName): The TemplateName of the template
            class being run.

    Returns:
        None
    """

    project_id: str
    _, project_id = google.auth.default()

    logger = spark.sparkContext._jvm.org.apache.log4j.LogManager.getLogger(__name__)

    try:
        client_info = http_client_info.ClientInfo(user_agent=f"google-pso-tool/dataproc-templates/0.1.0-{template_name.value}")
        client = bigquery.Client(project=project_id, client_info=client_info)
        dataset = client.get_dataset('bigquery-public-data.austin_311')
        logger.info("Tracked invocation: " + dataset.dataset_id)
    except Exception:
        # Do nothing
        pass


# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint

from pyspark.sql import SparkSession

from dataproc_templates import BaseTemplate
import dataproc_templates.util.template_constants as constants
from dataproc_templates.util.argument_parsing import add_spark_options
from dataproc_templates.util.dataframe_reader_wrappers import ingest_dataframe_from_cloud_storage


__all__ = ['GCSToJDBCTemplate']


class GCSToJDBCTemplate(BaseTemplate):
    """
    Dataproc template implementing loads from Cloud Storage into JDBC
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.GCS_JDBC_INPUT_LOCATION}',
            dest=constants.GCS_JDBC_INPUT_LOCATION,
            required=True,
            help='Cloud Storage location of the input files'
        )
        parser.add_argument(
            f'--{constants.GCS_JDBC_INPUT_FORMAT}',
            dest=constants.GCS_JDBC_INPUT_FORMAT,
            required=True,
            help='Input file format (one of: avro,parquet,csv,json,delta)',
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON,
                constants.FORMAT_DELTA
            ]
        )
        add_spark_options(parser, constants.get_csv_input_spark_options("gcs.jdbc.input."))
        parser.add_argument(
            f'--{constants.GCS_JDBC_OUTPUT_TABLE}',
            dest=constants.GCS_JDBC_OUTPUT_TABLE,
            required=True,
            help='JDBC output table name'
        )
        parser.add_argument(
            f'--{constants.GCS_JDBC_OUTPUT_MODE}',
            dest=constants.GCS_JDBC_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        parser.add_argument(
            f'--{constants.GCS_JDBC_OUTPUT_URL}',
            dest=constants.GCS_JDBC_OUTPUT_URL,
            required=True,
            help='JDBC output URL'
        )
        parser.add_argument(
            f'--{constants.GCS_JDBC_OUTPUT_DRIVER}',
            dest=constants.GCS_JDBC_OUTPUT_DRIVER,
            required=True,
            help='JDBC output driver name'
        )
        parser.add_argument(
            f'--{constants.GCS_JDBC_BATCH_SIZE}',
            dest=constants.GCS_JDBC_BATCH_SIZE,
            required=False,
            default=1000,
            help='JDBC output batch size'
        )
        parser.add_argument(
            f'--{constants.GCS_JDBC_NUMPARTITIONS}',
            dest=constants.GCS_JDBC_NUMPARTITIONS,
            required=False,
            help='The maximum number of partitions to be used for parallelism in table writing'
        )

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:
        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_location: str = args[constants.GCS_JDBC_INPUT_LOCATION]
        input_format: str = args[constants.GCS_JDBC_INPUT_FORMAT]
        jdbc_url: str = args[constants.GCS_JDBC_OUTPUT_URL]
        jdbc_table: str = args[constants.GCS_JDBC_OUTPUT_TABLE]
        output_mode: str = args[constants.GCS_JDBC_OUTPUT_MODE]
        output_driver: str = args[constants.GCS_JDBC_OUTPUT_DRIVER]
        batch_size: int = args[constants.GCS_JDBC_BATCH_SIZE]
        jdbc_numpartitions: int = args[constants.GCS_JDBC_NUMPARTITIONS]

        logger.info(
            "Starting Cloud Storage to JDBC Spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        # Read
        input_data = ingest_dataframe_from_cloud_storage(spark, args, input_location, input_format, "gcs.jdbc.input.")

        # Write
        if not jdbc_numpartitions:
            jdbc_numpartitions = input_data.rdd.getNumPartitions()

        # TODO Convert this call to a function in dataproc_templates.util.dataframe_writer_wrappers
        input_data.write \
            .format(constants.FORMAT_JDBC) \
            .option(constants.JDBC_URL, jdbc_url) \
            .option(constants.JDBC_TABLE, jdbc_table) \
            .option(constants.JDBC_DRIVER, output_driver) \
            .option(constants.JDBC_BATCH_SIZE, batch_size) \
            .option(constants.JDBC_NUMPARTITIONS, jdbc_numpartitions) \
            .mode(output_mode) \
            .save()



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint
import sys
from pyspark.sql import SparkSession, DataFrameWriter

from dataproc_templates import BaseTemplate
from dataproc_templates.util.argument_parsing import add_spark_options
from dataproc_templates.util.dataframe_writer_wrappers import persist_dataframe_to_cloud_storage
import dataproc_templates.util.template_constants as constants


__all__ = ['HiveToGCSTemplate']


class HiveToGCSTemplate(BaseTemplate):
    """
    Dataproc template implementing exports from Hive to GCS
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.HIVE_GCS_INPUT_DATABASE}',
            dest=constants.HIVE_GCS_INPUT_DATABASE,
            required=True,
            help='Hive database for exporting data to GCS'
        )

        parser.add_argument(
            f'--{constants.HIVE_GCS_INPUT_TABLE}',
            dest=constants.HIVE_GCS_INPUT_TABLE,
            required=True,
            help='Hive table for exporting data to GCS'
        )
        parser.add_argument(
            f'--{constants.HIVE_GCS_OUTPUT_LOCATION}',
            dest=constants.HIVE_GCS_OUTPUT_LOCATION,
            required=True,
            help='Cloud Storage location for output files'
        )
        parser.add_argument(
            f'--{constants.HIVE_GCS_OUTPUT_FORMAT}',
            dest=constants.HIVE_GCS_OUTPUT_FORMAT,
            required=False,
            default=constants.FORMAT_PRQT,
            help=(
                'Output file format '
                '(one of: avro,parquet,csv,json) '
                '(Defaults to parquet)'
            ),
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON
            ]
        )
        parser.add_argument(
            f'--{constants.HIVE_GCS_OUTPUT_MODE}',
            dest=constants.HIVE_GCS_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_OVERWRITE,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to overwrite)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        parser.add_argument(
            f'--{constants.HIVE_GCS_TEMP_VIEW_NAME}',
            dest=constants.HIVE_GCS_TEMP_VIEW_NAME,
            required=False,
            default="",
            help='Temp view name for creating a spark sql view on source data. This name has to match with the table name that will be used in the SQL query'
        )
        parser.add_argument(
            f'--{constants.HIVE_GCS_SQL_QUERY}',
            dest=constants.HIVE_GCS_SQL_QUERY,
            required=False,
            default="",
            help='SQL query for data transformation. This must use the temp view name as the table to query from.'
        )
        add_spark_options(parser, constants.get_csv_output_spark_options("hive.gcs.output."))

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        if getattr(known_args, constants.HIVE_GCS_SQL_QUERY) and not getattr(known_args, constants.HIVE_GCS_TEMP_VIEW_NAME):
            sys.exit('ArgumentParser Error: Temp view name cannot be null if you want to do data transformations with query')

        return vars(known_args)

    
    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        hive_database: str = args[constants.HIVE_GCS_INPUT_DATABASE]
        hive_table: str = args[constants.HIVE_GCS_INPUT_TABLE]
        output_location: str = args[constants.HIVE_GCS_OUTPUT_LOCATION]
        output_format: str = args[constants.HIVE_GCS_OUTPUT_FORMAT]
        output_mode: str = args[constants.HIVE_GCS_OUTPUT_MODE]
        hive_temp_view: str = args[constants.HIVE_GCS_TEMP_VIEW_NAME]
        sql_query: str = args[constants.HIVE_GCS_SQL_QUERY]

        logger.info(
            "Starting Hive to GCS spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        # Read
        input_data = spark.table(hive_database + "." + hive_table)
      
        if sql_query:
            # Create temp view on source data
            input_data.createGlobalTempView(hive_temp_view)
            # Execute SQL
            output_data = spark.sql(sql_query)
        else:
            output_data = input_data

        writer: DataFrameWriter = output_data.write.mode(output_mode)
        persist_dataframe_to_cloud_storage(writer, args, output_location, output_format, "hive.gcs.output.")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.



from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint
from pyspark.sql import SparkSession, DataFrame, DataFrameWriter
from dataproc_templates import BaseTemplate
import dataproc_templates.util.template_constants as constants


__all__ = ['AzureBlobStorageToBigQueryTemplate']


class AzureBlobStorageToBigQueryTemplate(BaseTemplate):
    """
    Dataproc template implementing loads from GCS into BigQuery
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.AZ_BLOB_BQ_INPUT_LOCATION}',
            dest=constants.AZ_BLOB_BQ_INPUT_LOCATION,
            required=True,
            help='Location of the input files'
        )
        parser.add_argument(
            f'--{constants.AZ_BLOB_BQ_OUTPUT_DATASET}',
            dest=constants.AZ_BLOB_BQ_OUTPUT_DATASET,
            required=True,
            help='BigQuery dataset for the output table'
        )
        parser.add_argument(
            f'--{constants.AZ_BLOB_BQ_OUTPUT_TABLE}',
            dest=constants.AZ_BLOB_BQ_OUTPUT_TABLE,
            required=True,
            help='BigQuery output table name'
        )
        parser.add_argument(
            f'--{constants.AZ_BLOB_BQ_INPUT_FORMAT}',
            dest=constants.AZ_BLOB_BQ_INPUT_FORMAT,
            required=True,
            help='Input file format (one of: avro,parquet,csv,json)',
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON,
                constants.FORMAT_DELTA
            ]
        )
        parser.add_argument(
            f'--{constants.AZ_BLOB_BQ_LD_TEMP_BUCKET_NAME}',
            dest=constants.AZ_BLOB_BQ_LD_TEMP_BUCKET_NAME,
            required=True,
            help='Spark BigQuery connector temporary bucket'
        )
        parser.add_argument(
            f'--{constants.AZ_BLOB_BQ_OUTPUT_MODE}',
            dest=constants.AZ_BLOB_BQ_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        parser.add_argument(
            f'--{constants.AZ_BLOB_STORAGE_ACCOUNT}',
            dest=constants.AZ_BLOB_STORAGE_ACCOUNT,
            required=True,
            help='Azure Storage Account'
        )
        parser.add_argument(
            f'--{constants.AZ_BLOB_CONTAINER_NAME}',
            dest=constants.AZ_BLOB_CONTAINER_NAME,
            required=True,
            help='Azure Account Name'
        )

        parser.add_argument(
            f'--{constants.AZ_BLOB_SAS_TOKEN}',
            dest=constants.AZ_BLOB_SAS_TOKEN,
            required=True,
            help='Azure SAS TOKEN'
        )

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_file_location: str = args[constants.AZ_BLOB_BQ_INPUT_LOCATION]
        big_query_dataset: str = args[constants.AZ_BLOB_BQ_OUTPUT_DATASET]
        big_query_table: str = args[constants.AZ_BLOB_BQ_OUTPUT_TABLE]
        input_file_format: str = args[constants.AZ_BLOB_BQ_INPUT_FORMAT]
        bq_temp_bucket: str = args[constants.AZ_BLOB_BQ_LD_TEMP_BUCKET_NAME]
        output_mode: str = args[constants.AZ_BLOB_BQ_OUTPUT_MODE]
        storage_account: str = args[constants.AZ_BLOB_STORAGE_ACCOUNT]
        container_name: str = args[constants.AZ_BLOB_CONTAINER_NAME]
        sas_token: str = args[constants.AZ_BLOB_SAS_TOKEN]

        spark.conf.set(f"fs.azure.sas.{container_name}.{storage_account}.blob.core.windows.net", sas_token)

        logger.info(f"Starting Azure to BigQuery spark job with parameters:\n {pprint.pformat(args)}")

        # Read
        input_data: DataFrame

        if input_file_format == constants.FORMAT_PRQT:
            input_data = spark.read \
                .parquet(input_file_location)
        elif input_file_format == constants.FORMAT_AVRO:
            input_data = spark.read \
                .format(constants.FORMAT_AVRO_EXTD) \
                .load(input_file_location)
        elif input_file_format == constants.FORMAT_CSV:
            input_data = spark.read \
                .format(constants.FORMAT_CSV) \
                .option(constants.HEADER, True) \
                .option(constants.INFER_SCHEMA, True) \
                .load(input_file_location)
        elif input_file_format == constants.FORMAT_JSON:
            input_data = spark.read \
                .json(input_file_location)
        elif input_file_format == constants.FORMAT_DELTA:
            input_data = spark.read \
                .format(constants.FORMAT_DELTA) \
                .load(input_file_location)

        # Write
        input_data.write \
            .format(constants.FORMAT_BIGQUERY) \
            .option(constants.TABLE, big_query_dataset + "." + big_query_table) \
            .option(constants.AZ_BLOB_BQ_TEMP_BUCKET, bq_temp_bucket) \
            .mode(output_mode) \
            .save()



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint

from pyspark.sql import SparkSession, DataFrame

from dataproc_templates import BaseTemplate
import dataproc_templates.util.template_constants as constants

__all__ = ['S3ToBigQueryTemplate']


class S3ToBigQueryTemplate(BaseTemplate):

    """
    Dataproc template implementing exports from Amazon S3 to BigQuery
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.S3_BQ_INPUT_LOCATION}',
            dest=constants.S3_BQ_INPUT_LOCATION,
            required=True,
            help='Amazon S3 input location. Input location must begin with s3a://'
        )

        parser.add_argument(
            f'--{constants.S3_BQ_ACCESS_KEY}',
            dest=constants.S3_BQ_ACCESS_KEY,
            required=True,
            help='Access key to access Amazon S3 bucket'
        )

        parser.add_argument(
            f'--{constants.S3_BQ_SECRET_KEY}',
            dest=constants.S3_BQ_SECRET_KEY,
            required=True,
            help='Secret key to access Amazon S3 bucket'
        )

        parser.add_argument(
            f'--{constants.S3_BQ_INPUT_FORMAT}',
            dest=constants.S3_BQ_INPUT_FORMAT,
            required=True,
            help='Input file format in Amazon S3 bucket (one of : avro, parquet, csv, json)',
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON
            ]
        )

        parser.add_argument(
            f'--{constants.S3_BQ_OUTPUT_DATASET_NAME}',
            dest=constants.S3_BQ_OUTPUT_DATASET_NAME,
            required=True,
            help='BigQuery dataset for the output table'
        )

        parser.add_argument(
            f'--{constants.S3_BQ_OUTPUT_TABLE_NAME}',
            dest=constants.S3_BQ_OUTPUT_TABLE_NAME,
            required=True,
            help='BigQuery output table name'
        )

        parser.add_argument(
            f'--{constants.S3_BQ_TEMP_BUCKET_NAME}',
            dest=constants.S3_BQ_TEMP_BUCKET_NAME,
            required=True,
            help='Pre existing GCS bucket name where temporary files are staged'
        )

        parser.add_argument(
            f'--{constants.S3_BQ_OUTPUT_MODE}',
            dest=constants.S3_BQ_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists)'
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_file_location: str = args[constants.S3_BQ_INPUT_LOCATION]
        access_key: str = args[constants.S3_BQ_ACCESS_KEY]
        secret_key: str = args[constants.S3_BQ_SECRET_KEY]
        input_file_format: str = args[constants.S3_BQ_INPUT_FORMAT]
        bq_dataset: str = args[constants.S3_BQ_OUTPUT_DATASET_NAME]
        bq_table: str = args[constants.S3_BQ_OUTPUT_TABLE_NAME]
        bq_temp_bucket: str = args[constants.S3_BQ_TEMP_BUCKET_NAME]
        output_mode: str = args[constants.S3_BQ_OUTPUT_MODE]

        ignore_keys = {constants.S3_BQ_ACCESS_KEY, constants.S3_BQ_SECRET_KEY}
        filtered_args = {key:val for key,val in args.items() if key not in ignore_keys}
        logger.info(
            "Starting Amazon S3 to Bigquery spark job with parameters:\n"
            f"{pprint.pformat(filtered_args)}"
        )

        # Set configuration to connect to Amazon S3
        spark._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ENDPOINT, constants.S3_BQ_ENDPOINT_VALUE)
        spark._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3ACCESSKEY, access_key)
        spark._jsc.hadoopConfiguration() \
            .set(constants.AWS_S3SECRETKEY, secret_key)

        # Read
        input_data: DataFrame

        if input_file_format == constants.FORMAT_PRQT:
            input_data = spark.read \
                .parquet(input_file_location)
        elif input_file_format == constants.FORMAT_AVRO:
            input_data = spark.read \
                .format(constants.FORMAT_AVRO_EXTD) \
                .load(input_file_location)
        elif input_file_format == constants.FORMAT_CSV:
            input_data = spark.read \
                .format(constants.FORMAT_CSV) \
                .option(constants.CSV_HEADER, True) \
                .option(constants.CSV_INFER_SCHEMA, True) \
                .load(input_file_location)
        elif input_file_format == constants.FORMAT_JSON:
            input_data = spark.read \
                .json(input_file_location)

        # Write
        input_data.write \
            .format(constants.FORMAT_BIGQUERY) \
            .option(constants.TABLE, bq_dataset + "." + bq_table) \
            .option(constants.TEMP_GCS_BUCKET, bq_temp_bucket) \
            .mode(output_mode) \
            .save()



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.hive.hive_to_bigquery import HiveToBigQueryTemplate
import dataproc_templates.util.template_constants as constants


class TestHiveToBigQueryTemplate:
    """
    Test suite for HiveToBigQueryTemplate
    """

    def test_parse_args(self):
        """Tests HiveToBigQueryTemplate.parse_args()"""

        hive_to_bigquery_template = HiveToBigQueryTemplate()
        parsed_args = hive_to_bigquery_template.parse_args(
            ["--hive.bigquery.input.database=database",
            "--hive.bigquery.input.table=table",
             "--hive.bigquery.output.dataset=dataset",
             "--hive.bigquery.output.table=table",
             "--hive.bigquery.temp.bucket.name=bucket",
             "--hive.bigquery.output.mode=overwrite",
             "--hive.bigquery.temp.view.name=temp",
             "--hive.bigquery.sql.query='select * from temp'"])

        assert parsed_args["hive.bigquery.input.database"] == "database"
        assert parsed_args["hive.bigquery.input.table"] == "table"
        assert parsed_args["hive.bigquery.output.dataset"] == "dataset"
        assert parsed_args["hive.bigquery.output.table"] == "table"
        assert parsed_args["hive.bigquery.temp.bucket.name"] == "bucket"
        assert parsed_args["hive.bigquery.output.mode"] == "overwrite"
        assert parsed_args["hive.bigquery.temp.view.name"] == "temp"
        assert parsed_args["hive.bigquery.sql.query"] == "'select * from temp'"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_overwrite(self, mock_spark_session):
        """Tests HiveToBigQueryTemplate runs with overwrite mode"""

        hive_to_bigquery_template = HiveToBigQueryTemplate()
        mock_parsed_args = hive_to_bigquery_template.parse_args(
            ["--hive.bigquery.input.database=database",
            "--hive.bigquery.input.table=table",
             "--hive.bigquery.output.dataset=dataset",
             "--hive.bigquery.output.table=table",
             "--hive.bigquery.temp.bucket.name=bucket",
             "--hive.bigquery.output.mode=overwrite"])
        mock_spark_session.table.return_value = mock_spark_session.dataframe.DataFrame
        hive_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.table.assert_called_once_with("database.table")
        mock_spark_session.dataframe.DataFrame.write \
            .format.assert_called_once_with(constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option.assert_called_once_with(constants.TEMP_GCS_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode() \
            .save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_append(self, mock_spark_session):
        """Tests HiveToBigQueryTemplate runs with append mode"""

        hive_to_bigquery_template = HiveToBigQueryTemplate()
        mock_parsed_args = hive_to_bigquery_template.parse_args(
            ["--hive.bigquery.input.database=database",
            "--hive.bigquery.input.table=table",
             "--hive.bigquery.output.dataset=dataset",
             "--hive.bigquery.output.table=table",
             "--hive.bigquery.temp.bucket.name=bucket",
             "--hive.bigquery.output.mode=append"])
        mock_spark_session.table.return_value = mock_spark_session.dataframe.DataFrame
        hive_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.table.assert_called_once_with("database.table")
        mock_spark_session.dataframe.DataFrame.write \
            .format.assert_called_once_with(constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option.assert_called_once_with(constants.TEMP_GCS_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write \
            .format() \
            .option() \
            .option() \
            .mode() \
            .save.assert_called_once()



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.redshift.redshift_to_gcs import RedshiftToGCSTemplate
import dataproc_templates.util.template_constants as constants


class TestRedshiftToGCSTemplate:
    """
    Test suite for RedshiftToGCSTemplate
    """

    def test_parse_args1(self):
        """Tests RedshiftToGCSTemplate.parse_args()"""

        redshift_to_gcs_template = RedshiftToGCSTemplate()
        parsed_args = redshift_to_gcs_template.parse_args(
            ["--redshifttogcs.input.url=url",
             "--redshifttogcs.s3.tempdir=s3a://temp",
             "--redshifttogcs.input.table=table1",
             "--redshifttogcs.iam.rolearn=arn:aws:iam::xxxx:role/role",
             "--redshifttogcs.s3.accesskey=xyz",
             "--redshifttogcs.s3.secretkey=lmn",
             "--redshifttogcs.output.location=gs://test",
             "--redshifttogcs.output.format=csv",
             "--redshifttogcs.output.mode=append",
             "--redshifttogcs.output.partitioncolumn=column"
             ])

        assert parsed_args["redshifttogcs.input.url"] == "url"
        assert parsed_args["redshifttogcs.s3.tempdir"] == "s3a://temp"
        assert parsed_args["redshifttogcs.input.table"] == "table1"
        assert parsed_args["redshifttogcs.iam.rolearn"] == "arn:aws:iam::xxxx:role/role"
        assert parsed_args["redshifttogcs.s3.accesskey"] == "xyz"
        assert parsed_args["redshifttogcs.s3.secretkey"] == "lmn"
        assert parsed_args["redshifttogcs.output.location"] == "gs://test"
        assert parsed_args["redshifttogcs.output.format"] == "csv"
        assert parsed_args["redshifttogcs.output.mode"] == "append"
        assert parsed_args["redshifttogcs.output.partitioncolumn"] == "column"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args2(self, mock_spark_session):
        """Tests RedshiftToGCSTemplate write parquet"""

        redshift_to_gcs_template = RedshiftToGCSTemplate()

        mock_parsed_args = redshift_to_gcs_template.parse_args(
            ["--redshifttogcs.input.url=url",
             "--redshifttogcs.s3.tempdir=s3a://temp",
             "--redshifttogcs.input.table=table1",
             "--redshifttogcs.iam.rolearn=arn:aws:iam::xxxx:role/role",
             "--redshifttogcs.s3.accesskey=xyz",
             "--redshifttogcs.s3.secretkey=lmn",
             "--redshifttogcs.output.location=gs://test",
             "--redshifttogcs.output.format=parquet",
             "--redshifttogcs.output.mode=overwrite"
             ])
        mock_spark_session._jsc.hadoopConfiguration().set(constants.AWS_S3ACCESSKEY, "xyz")
        mock_spark_session._jsc.hadoopConfiguration().set(constants.AWS_S3SECRETKEY, "lmn")
        mock_spark_session.read.format().option().option().option().option().load.return_value = mock_spark_session.dataframe.DataFrame
        redshift_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_REDSHIFT)
        mock_spark_session.read.format().option.assert_called_with(constants.JDBC_URL, "url")
        mock_spark_session.read.format().option().option.assert_called_with(constants.JDBC_TABLE, "table1")
        mock_spark_session.read.format().option().option().option.assert_called_with(constants.REDSHIFT_TEMPDIR, "s3a://temp")
        mock_spark_session.read.format().option().option().option().option.assert_called_with(constants.REDSHIFT_IAMROLE, "arn:aws:iam::xxxx:role/role")
        mock_spark_session.read.format().option().option().option().option().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.dataframe.DataFrame.write.mode().parquet.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args3(self, mock_spark_session):
        """Tests RedshiftToGCSTemplate write avro"""

        redshift_to_gcs_template = RedshiftToGCSTemplate()

        mock_parsed_args = redshift_to_gcs_template.parse_args(
            ["--redshifttogcs.input.url=url",
             "--redshifttogcs.s3.tempdir=s3a://temp",
             "--redshifttogcs.input.table=table1",
             "--redshifttogcs.iam.rolearn=arn:aws:iam::xxxx:role/role",
             "--redshifttogcs.s3.accesskey=xyz",
             "--redshifttogcs.s3.secretkey=lmn",
             "--redshifttogcs.output.location=gs://test",
             "--redshifttogcs.output.format=avro",
             "--redshifttogcs.output.mode=append"
             ])
        mock_spark_session.read.format().option().option().option().option().load.return_value = mock_spark_session.dataframe.DataFrame
        redshift_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_REDSHIFT)
        mock_spark_session.read.format().option.assert_called_with(constants.JDBC_URL, "url")
        mock_spark_session.read.format().option().option.assert_called_with(constants.JDBC_TABLE, "table1")
        mock_spark_session.read.format().option().option().option.assert_called_with(constants.REDSHIFT_TEMPDIR, "s3a://temp")
        mock_spark_session.read.format().option().option().option().option.assert_called_with(constants.REDSHIFT_IAMROLE, "arn:aws:iam::xxxx:role/role")
        mock_spark_session.read.format().option().option().option().option().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.mode().format.assert_called_once_with(constants.FORMAT_AVRO)
        mock_spark_session.dataframe.DataFrame.write.mode().format().save.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args4(self, mock_spark_session):
        """Tests RedshiftToGCSTemplate write csv"""

        redshift_to_gcs_template = RedshiftToGCSTemplate()

        mock_parsed_args = redshift_to_gcs_template.parse_args(
            ["--redshifttogcs.input.url=url",
             "--redshifttogcs.s3.tempdir=s3a://temp",
             "--redshifttogcs.input.table=table1",
             "--redshifttogcs.iam.rolearn=arn:aws:iam::xxxx:role/role",
             "--redshifttogcs.s3.accesskey=xyz",
             "--redshifttogcs.s3.secretkey=lmn",
             "--redshifttogcs.output.location=gs://test",
             "--redshifttogcs.output.format=csv",
             "--redshifttogcs.output.mode=ignore"
             ])
        mock_spark_session.read.format().option().option().option().option().load.return_value = mock_spark_session.dataframe.DataFrame
        redshift_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_REDSHIFT)
        mock_spark_session.read.format().option.assert_called_with(constants.JDBC_URL, "url")
        mock_spark_session.read.format().option().option.assert_called_with(constants.JDBC_TABLE, "table1")
        mock_spark_session.read.format().option().option().option.assert_called_with(constants.REDSHIFT_TEMPDIR, "s3a://temp")
        mock_spark_session.read.format().option().option().option().option.assert_called_with(constants.REDSHIFT_IAMROLE, "arn:aws:iam::xxxx:role/role")
        mock_spark_session.read.format().option().option().option().option().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.mode().options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})
        mock_spark_session.dataframe.DataFrame.write.mode().options().csv.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args5(self, mock_spark_session):
        """Tests RedshiftToGCSTemplate write json"""

        redshift_to_gcs_template = RedshiftToGCSTemplate()

        mock_parsed_args = redshift_to_gcs_template.parse_args(
            ["--redshifttogcs.input.url=url",
             "--redshifttogcs.s3.tempdir=s3a://temp",
             "--redshifttogcs.input.table=table1",
             "--redshifttogcs.iam.rolearn=arn:aws:iam::xxxx:role/role",
             "--redshifttogcs.s3.accesskey=xyz",
             "--redshifttogcs.s3.secretkey=lmn",
             "--redshifttogcs.output.location=gs://test",
             "--redshifttogcs.output.format=json",
             "--redshifttogcs.output.mode=ignore"
             ])
        mock_spark_session.read.format().option().option().option().option().load.return_value = mock_spark_session.dataframe.DataFrame
        redshift_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_REDSHIFT)
        mock_spark_session.read.format().option.assert_called_with(constants.JDBC_URL, "url")
        mock_spark_session.read.format().option().option.assert_called_with(constants.JDBC_TABLE, "table1")
        mock_spark_session.read.format().option().option().option.assert_called_with(constants.REDSHIFT_TEMPDIR, "s3a://temp")
        mock_spark_session.read.format().option().option().option().option.assert_called_with(constants.REDSHIFT_IAMROLE, "arn:aws:iam::xxxx:role/role")
        mock_spark_session.read.format().option().option().option().option().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.mode().json.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args6(self, mock_spark_session):
        """Tests RedshiftToGCSTemplate pass args"""

        redshift_to_gcs_template = RedshiftToGCSTemplate()

        mock_parsed_args = redshift_to_gcs_template.parse_args(
            ["--redshifttogcs.input.url=url",
             "--redshifttogcs.s3.tempdir=s3a://temp",
             "--redshifttogcs.input.table=table1",
             "--redshifttogcs.iam.rolearn=arn:aws:iam::xxxx:role/role",
             "--redshifttogcs.s3.accesskey=xyz",
             "--redshifttogcs.s3.secretkey=lmn",
             "--redshifttogcs.output.location=gs://test",
             "--redshifttogcs.output.format=csv",
             "--redshifttogcs.output.mode=append"
             ])
        mock_spark_session.read.format().option().option().option().option().load.return_value = mock_spark_session.dataframe.DataFrame
        redshift_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_REDSHIFT)
        mock_spark_session.read.format().option.assert_called_with(constants.JDBC_URL, "url")
        mock_spark_session.read.format().option().option.assert_called_with(constants.JDBC_TABLE, "table1")
        mock_spark_session.read.format().option().option().option.assert_called_with(constants.REDSHIFT_TEMPDIR, "s3a://temp")
        mock_spark_session.read.format().option().option().option().option.assert_called_with(constants.REDSHIFT_IAMROLE, "arn:aws:iam::xxxx:role/role")
        mock_spark_session.read.format().option().option().option().option().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.mode().options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})
        mock_spark_session.dataframe.DataFrame.write.mode().options().csv.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args7(self, mock_spark_session):
        """Tests RedshiftToGCSTemplate pass args"""

        redshift_to_gcs_template = RedshiftToGCSTemplate()

        mock_parsed_args = redshift_to_gcs_template.parse_args(
            ["--redshifttogcs.input.url=url",
             "--redshifttogcs.s3.tempdir=s3a://temp",
             "--redshifttogcs.input.table=table1",
             "--redshifttogcs.iam.rolearn=arn:aws:iam::xxxx:role/role",
             "--redshifttogcs.s3.accesskey=xyz",
             "--redshifttogcs.s3.secretkey=lmn",
             "--redshifttogcs.output.location=gs://test",
             "--redshifttogcs.output.format=csv",
             "--redshifttogcs.output.mode=append",
             "--redshifttogcs.output.partitioncolumn=column"
             ])
        mock_spark_session.read.format().option().option().option().option().load.return_value = mock_spark_session.dataframe.DataFrame
        redshift_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_REDSHIFT)
        mock_spark_session.read.format().option.assert_called_with(constants.JDBC_URL, "url")
        mock_spark_session.read.format().option().option.assert_called_with(constants.JDBC_TABLE, "table1")
        mock_spark_session.read.format().option().option().option.assert_called_with(constants.REDSHIFT_TEMPDIR, "s3a://temp")
        mock_spark_session.read.format().option().option().option().option.assert_called_with(constants.REDSHIFT_IAMROLE, "arn:aws:iam::xxxx:role/role")
        mock_spark_session.read.format().option().option().option().option().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.mode().partitionBy.assert_called_once_with("column")
        mock_spark_session.dataframe.DataFrame.write.mode().partitionBy().options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})
        mock_spark_session.dataframe.DataFrame.write.mode().partitionBy().options().csv.assert_called_once_with("gs://test")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint

from pyspark.sql import SparkSession, DataFrameWriter

from dataproc_templates import BaseTemplate
from dataproc_templates.util.argument_parsing import add_spark_options
from dataproc_templates.util.dataframe_writer_wrappers import persist_dataframe_to_cloud_storage
import dataproc_templates.util.template_constants as constants


__all__ = ['MongoToGCSTemplate']


class MongoToGCSTemplate(BaseTemplate):
    """
    Dataproc template implementing exports from Mongo to Cloud Storage
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.MONGO_GCS_INPUT_URI}',
            dest=constants.MONGO_GCS_INPUT_URI,
            required=True,
            help='MONGO Cloud Storage Input Connection Uri'
        )
        parser.add_argument(
            f'--{constants.MONGO_GCS_INPUT_DATABASE}',
            dest=constants.MONGO_GCS_INPUT_DATABASE,
            required=True,
            help='MONGO Cloud Storage Input Database Name'
        )
        parser.add_argument(
            f'--{constants.MONGO_GCS_INPUT_COLLECTION}',
            dest=constants.MONGO_GCS_INPUT_COLLECTION,
            required=True,
            help='MONGO Cloud Storage Input Collection Name'
        )
        parser.add_argument(
            f'--{constants.MONGO_GCS_OUTPUT_FORMAT}',
            dest=constants.MONGO_GCS_OUTPUT_FORMAT,
            required=True,
            help='Output file format (one of: avro,parquet,csv,json)',
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON
            ]
        )
        parser.add_argument(
            f'--{constants.MONGO_GCS_OUTPUT_LOCATION}',
            dest=constants.MONGO_GCS_OUTPUT_LOCATION,
            required=True,
            help='Cloud Storage location for output files'
        )
        parser.add_argument(
            f'--{constants.MONGO_GCS_OUTPUT_MODE}',
            dest=constants.MONGO_GCS_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        add_spark_options(parser, constants.get_csv_output_spark_options("mongo.gcs.output."))

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_uri: str = args[constants.MONGO_GCS_INPUT_URI]
        input_database: str = args[constants.MONGO_GCS_INPUT_DATABASE]
        input_collection: str = args[constants.MONGO_GCS_INPUT_COLLECTION]
        output_format: str = args[constants.MONGO_GCS_OUTPUT_FORMAT]
        output_mode: str = args[constants.MONGO_GCS_OUTPUT_MODE]
        output_location: str = args[constants.MONGO_GCS_OUTPUT_LOCATION]

        logger.info(
            "Starting Mongo to Cloud Storage Spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        # Read
        input_data = spark.read \
            .format(constants.FORMAT_MONGO) \
            .option(constants.MONGO_INPUT_URI, input_uri) \
            .option(constants.MONGO_DATABASE, input_database) \
            .option(constants.MONGO_COLLECTION, input_collection) \
            .load()

        # Write
        writer: DataFrameWriter = input_data.write.mode(output_mode)
        persist_dataframe_to_cloud_storage(writer, args, output_location, output_format, "mongo.gcs.output.")


"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.jdbc.jdbc_to_bigquery import JDBCToBigQueryTemplate
import dataproc_templates.util.template_constants as constants


class TestJDBCToBigQueryTemplate:
    """
    Test suite for JDBCToBigQueryTemplate
    """

    def test_parse_args1(self):
        """Tests JDBCToBigQueryTemplate.parse_args() with partition column"""

        jdbc_to_bigquery_template = JDBCToBigQueryTemplate()
        parsed_args = jdbc_to_bigquery_template.parse_args(
            ["--jdbc.bigquery.input.url=url",
             "--jdbc.bigquery.input.driver=driver",
             "--jdbc.bigquery.input.table=table1",
             "--jdbc.bigquery.input.partitioncolumn=column",
             "--jdbc.bigquery.input.lowerbound=1",
             "--jdbc.bigquery.input.upperbound=2",
             "--jdbc.bigquery.numpartitions=5",
             "--jdbc.bigquery.input.fetchsize=100",
             "--jdbc.bigquery.input.sessioninitstatement=EXEC some_setup_sql('data1')",
             "--jdbc.bigquery.output.mode=append",
             "--jdbc.bigquery.output.dataset=bq-dataset",
             "--jdbc.bigquery.output.table=bq-table",
             "--jdbc.bigquery.temp.bucket.name=bucket-name",
             ])

        assert parsed_args["jdbc.bigquery.input.url"] == "url"
        assert parsed_args["jdbc.bigquery.input.driver"] == "driver"
        assert parsed_args["jdbc.bigquery.input.table"] == "table1"
        assert parsed_args["jdbc.bigquery.input.partitioncolumn"] == "column"
        assert parsed_args["jdbc.bigquery.input.lowerbound"] == "1"
        assert parsed_args["jdbc.bigquery.input.upperbound"] == "2"
        assert parsed_args["jdbc.bigquery.numpartitions"] == "5"
        assert parsed_args["jdbc.bigquery.input.fetchsize"] == 100
        assert parsed_args["jdbc.bigquery.input.sessioninitstatement"] == "EXEC some_setup_sql('data1')"
        assert parsed_args["jdbc.bigquery.output.mode"] == "append"
        assert parsed_args["jdbc.bigquery.output.dataset"] == "bq-dataset"
        assert parsed_args["jdbc.bigquery.output.table"] == "bq-table"
        assert parsed_args["jdbc.bigquery.temp.bucket.name"] == "bucket-name"

    def test_run_pass_args2(self):
        """Tests JDBCToBigQueryTemplate.parse_args() without partition column"""

        jdbc_to_bigquery_template = JDBCToBigQueryTemplate()

        parsed_args = jdbc_to_bigquery_template.parse_args(
            ["--jdbc.bigquery.input.url=url",
             "--jdbc.bigquery.input.driver=driver",
             "--jdbc.bigquery.input.table=table1",
             "--jdbc.bigquery.input.fetchsize=200",
             "--jdbc.bigquery.input.sessioninitstatement=EXEC some_setup_sql('data2')",
             "--jdbc.bigquery.output.mode=append",
             "--jdbc.bigquery.output.dataset=bq-dataset",
             "--jdbc.bigquery.output.table=bq-table",
             "--jdbc.bigquery.temp.bucket.name=bucket-name",
             ])

        assert parsed_args["jdbc.bigquery.input.url"] == "url"
        assert parsed_args["jdbc.bigquery.input.driver"] == "driver"
        assert parsed_args["jdbc.bigquery.input.table"] == "table1"
        assert parsed_args["jdbc.bigquery.input.fetchsize"] == 200
        assert parsed_args["jdbc.bigquery.input.sessioninitstatement"] == "EXEC some_setup_sql('data2')"
        assert parsed_args["jdbc.bigquery.output.mode"] == "append"
        assert parsed_args["jdbc.bigquery.output.dataset"] == "bq-dataset"
        assert parsed_args["jdbc.bigquery.output.table"] == "bq-table"
        assert parsed_args["jdbc.bigquery.temp.bucket.name"] == "bucket-name"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args3(self, mock_spark_session):
        """Tests JDBCToBigQueryTemplate pass args with partition column

        Intentionally left out sessionInitStatement which reduces the number of option() calls and therefore the number needed for mock.
        """

        jdbc_to_bigquery_template = JDBCToBigQueryTemplate()

        mock_parsed_args = jdbc_to_bigquery_template.parse_args(
            ["--jdbc.bigquery.input.url=url",
             "--jdbc.bigquery.input.driver=driver",
             "--jdbc.bigquery.input.table=table1",
             "--jdbc.bigquery.input.partitioncolumn=column",
             "--jdbc.bigquery.input.lowerbound=1",
             "--jdbc.bigquery.input.upperbound=2",
             "--jdbc.bigquery.numpartitions=5",
             "--jdbc.bigquery.output.mode=append",
             "--jdbc.bigquery.output.dataset=bq-dataset",
             "--jdbc.bigquery.output.table=bq-table",
             "--jdbc.bigquery.temp.bucket.name=bucket-name",
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_bigquery_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_JDBC)
        _, kwargs = mock_spark_session.read.format().options.call_args
        assert (constants.JDBC_URL, "url") in kwargs.items()
        assert (constants.JDBC_DRIVER, "driver") in kwargs.items()
        assert (constants.JDBC_TABLE, "table1") in kwargs.items()
        assert (constants.JDBC_PARTITIONCOLUMN, "column") in kwargs.items()
        assert (constants.JDBC_LOWERBOUND, "1") in kwargs.items()
        assert (constants.JDBC_UPPERBOUND, "2") in kwargs.items()
        assert (constants.JDBC_NUMPARTITIONS, "5") in kwargs.items()
        assert (constants.JDBC_FETCHSIZE, 0) in kwargs.items()
        assert constants.JDBC_SESSIONINITSTATEMENT not in kwargs
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "bq-dataset.bq-table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.JDBC_BQ_TEMP_BUCKET, "bucket-name")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args4(self, mock_spark_session):
        """Tests JDBCToBigQueryTemplate pass args without partition column"""

        jdbc_to_bigquery_template = JDBCToBigQueryTemplate()

        mock_parsed_args = jdbc_to_bigquery_template.parse_args(
            ["--jdbc.bigquery.input.url=url",
             "--jdbc.bigquery.input.driver=driver",
             "--jdbc.bigquery.input.table=table1",
             "--jdbc.bigquery.input.fetchsize=100",
             "--jdbc.bigquery.input.sessioninitstatement=EXEC some_setup_sql('data4')",
             "--jdbc.bigquery.output.mode=append",
             "--jdbc.bigquery.output.dataset=bq-dataset",
             "--jdbc.bigquery.output.table=bq-table",
             "--jdbc.bigquery.temp.bucket.name=bucket-name",
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_bigquery_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_JDBC)
        _, kwargs = mock_spark_session.read.format().options.call_args
        assert (constants.JDBC_URL, "url") in kwargs.items()
        assert (constants.JDBC_DRIVER, "driver") in kwargs.items()
        assert (constants.JDBC_TABLE, "table1") in kwargs.items()
        assert (constants.JDBC_NUMPARTITIONS, "10") in kwargs.items()
        assert (constants.JDBC_FETCHSIZE, 100) in kwargs.items()
        assert (constants.JDBC_SESSIONINITSTATEMENT, "EXEC some_setup_sql('data4')") in kwargs.items()
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "bq-dataset.bq-table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.JDBC_BQ_TEMP_BUCKET, "bucket-name")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.gcs.gcs_to_jdbc import GCSToJDBCTemplate
import dataproc_templates.util.template_constants as constants


class TestGCSToBigQueryTemplate:
    """
    Test suite for GCSToJDBCTemplate
    """

    def test_parse_args(self):
        """Tests GCSToJDBCTemplate.parse_args()"""

        gcs_to_jdbc_template = GCSToJDBCTemplate()
        parsed_args = gcs_to_jdbc_template.parse_args(
            ["--gcs.jdbc.input.format=parquet",
             "--gcs.jdbc.input.location=gs://test",
             "--gcs.jdbc.output.url=url",
             "--gcs.jdbc.output.table=table",
             "--gcs.jdbc.output.driver=driver",
             "--gcs.jdbc.output.mode=append"])

        assert parsed_args["gcs.jdbc.input.format"] == "parquet"
        assert parsed_args["gcs.jdbc.input.location"] == "gs://test"
        assert parsed_args["gcs.jdbc.output.table"] == "table"
        assert parsed_args["gcs.jdbc.output.mode"] == "append"
        assert parsed_args["gcs.jdbc.output.driver"] == "driver"
        assert parsed_args["gcs.jdbc.output.url"] == "url"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_parquet(self, mock_spark_session):
        """Tests GCSToJDBCTemplate runs with parquet format"""

        gcs_to_jdbc_template = GCSToJDBCTemplate()
        mock_parsed_args = gcs_to_jdbc_template.parse_args(
            ["--gcs.jdbc.input.format=parquet",
             "--gcs.jdbc.input.location=gs://test",
             "--gcs.jdbc.output.url=url",
             "--gcs.jdbc.output.table=table",
             "--gcs.jdbc.output.driver=driver",
             "--gcs.jdbc.output.mode=append"])
        mock_spark_session.read.parquet.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_jdbc_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.parquet.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option.assert_called_once_with(constants.JDBC_TABLE, "table")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_avro(self, mock_spark_session):
        """Tests GCSToJDBCTemplate runs with avro"""

        gcs_to_jdbc_template = GCSToJDBCTemplate()
        mock_parsed_args = gcs_to_jdbc_template.parse_args(
            ["--gcs.jdbc.input.format=avro",
             "--gcs.jdbc.input.location=gs://test",
             "--gcs.jdbc.output.url=url",
             "--gcs.jdbc.output.table=table",
             "--gcs.jdbc.output.driver=driver",
             "--gcs.jdbc.output.mode=append"])
        mock_spark_session.read.format().load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_jdbc_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_AVRO_EXTD)
        mock_spark_session.read.format().load.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option.assert_called_once_with(constants.JDBC_TABLE, "table")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_json(self, mock_spark_session):
        """Tests GCSToJDBCTemplate runs with json format"""

        gcs_to_jdbc_template = GCSToJDBCTemplate()
        mock_parsed_args = gcs_to_jdbc_template.parse_args(
            ["--gcs.jdbc.input.format=json",
             "--gcs.jdbc.input.location=gs://test",
             "--gcs.jdbc.output.url=url",
             "--gcs.jdbc.output.table=table",
             "--gcs.jdbc.output.driver=driver",
             "--gcs.jdbc.output.mode=append"])
        mock_spark_session.read.json.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_jdbc_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.json.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option.assert_called_once_with(constants.JDBC_TABLE, "table")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_delta(self, mock_spark_session):
        """Tests GCSToJDBCTemplate runs with delta"""

        gcs_to_jdbc_template = GCSToJDBCTemplate()
        mock_parsed_args = gcs_to_jdbc_template.parse_args(
            ["--gcs.jdbc.input.format=delta",
             "--gcs.jdbc.input.location=gs://test",
             "--gcs.jdbc.output.url=url",
             "--gcs.jdbc.output.table=table",
             "--gcs.jdbc.output.driver=driver",
             "--gcs.jdbc.output.mode=append"])
        mock_spark_session.read.format().load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_jdbc_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_DELTA)
        mock_spark_session.read.format().load.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option.assert_called_once_with(constants.JDBC_TABLE, "table")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_csv1(self, mock_spark_session):
        """Tests GCSToJDBCTemplate runs with csv format"""

        gcs_to_jdbc_template = GCSToJDBCTemplate()
        mock_parsed_args = gcs_to_jdbc_template.parse_args(
            ["--gcs.jdbc.input.format=csv",
             "--gcs.jdbc.input.location=gs://test",
             "--gcs.jdbc.input.header=false",
             "--gcs.jdbc.output.url=url",
             "--gcs.jdbc.output.table=table",
             "--gcs.jdbc.output.driver=driver",
             "--gcs.jdbc.output.mode=append"])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_jdbc_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'false',
            constants.CSV_INFER_SCHEMA: 'true',
        })
        mock_spark_session.read.format().options().load.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option.assert_called_once_with(constants.JDBC_TABLE, "table")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_csv2(self, mock_spark_session):
        """Tests GCSToJDBCTemplate runs with csv format and some optional csv options"""

        gcs_to_jdbc_template = GCSToJDBCTemplate()
        mock_parsed_args = gcs_to_jdbc_template.parse_args(
            ["--gcs.jdbc.input.format=csv",
             "--gcs.jdbc.input.location=gs://test",
             "--gcs.jdbc.input.inferschema=false",
             "--gcs.jdbc.input.sep=|",
             "--gcs.jdbc.input.comment=#",
             "--gcs.jdbc.input.timestampntzformat=yyyy-MM-dd'T'HH:mm:ss",
             "--gcs.jdbc.output.url=url",
             "--gcs.jdbc.output.table=table",
             "--gcs.jdbc.output.driver=driver",
             "--gcs.jdbc.output.mode=append"])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_jdbc_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'true',
            constants.CSV_INFER_SCHEMA: 'false',
            constants.CSV_SEP: "|",
            constants.CSV_COMMENT: "#",
            constants.CSV_TIMESTAMPNTZFORMAT: "yyyy-MM-dd'T'HH:mm:ss",
        })
        mock_spark_session.read.format().options().load.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option.assert_called_once_with(constants.JDBC_TABLE, "table")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().option().option().option().mode().save.assert_called_once()

